close all
clear
clc

%This script simply plots some selected experimental values that are
%gathered in an excel sheet.
%Written by Sander K. Arstad in spring 2022

%% Div.
filename = "ExData.xlsx";%   sheet = 1;  xlRange = "A3:D14";
sheet = readtable(filename, "Sheet", "OO2", 'Range', 'B3:V37');

%--------% %%% Making it easier to access the data inherent in sheet
for i = 1:size(sheet, 1) %Used to find how many data-values are relevant 
    %-and stored in sheet
    if isnan(sheet{i, 3})
        dataValues = i - 1; %The length of the first table
        break;
    end %if isnan(sheet(i,3))
end %for i = 1:size(sheet, 1)


%Writing up so data is easy-to-reach
[RPM, DV_pos, Q, Q_Or1, Q_Or2, Pr, m, m_Or1, m_Or2, P_bar, T_bar, Humid,...
    DP250_1, DP25_1, DPSmar_1, DP250_2, DP50_2, DP25_2, DPSmar_2]...
    =readDataTable(filename, "OO2", 'B3:V37', 1, 12); %Determines which cells to look at

%% Compressor curve, 9 000 RPM
%--------% 
compressorCurve = 0;
if compressorCurve == 1

figure %plotting selected curve
funcName = "poly2"; %which degree the polynomial is. "poly2" = second degree.
[Q_fit, gof] = fit(Q, Pr, funcName);
[Q_Or1_fit, gof_Or1] = fit(Q_Or1, Pr, funcName);
[Q_Or2_fit, gof_Or2] = fit(Q_Or2, Pr, funcName);
p = plot(Q_fit, Q, Pr, 'x');
%axis([0.2 1.8 1 1.3]);
hold on; grid on; grid minor
%title("Q, mean");  
x = linspace(min(Q), max(Q), 100);
y = x.^2 .* Q_fit.p1 + x .* Q_fit.p2 + Q_fit.p3;
%plot(x,y, 'r--')
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
legName = ["Static samples", "Regression curve", "Predicted curve"];
legend(legName, 'location', 'best')


    %Quadratic regression line (and function)


figure %Plotting Or.1 response
plot(Q_Or1_fit, Q_Or1, Pr, 'x')
hold on; grid on; grid minor
title("Q, Or.1");  
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
axis([0 0.8 1 1.3]);

figure %Plotting Or.2 response
plot(Q_Or2_fit, Q_Or2, Pr, 'x')
hold on; grid on; grid minor
title("Q, Or.2");  
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
axis([0 0.8 1 1.3]);

figure %plotting Or2 on top of Or1
x_Or2 = linspace(min(Q_Or2), max(Q_Or2), length(Q_Or2));
y_Or2 = x_Or2.^2 .* Q_Or2_fit.p1 + x_Or2 .* Q_Or2_fit.p2 + Q_Or2_fit.p3;

plot(Q_Or1_fit, Q_Or1, Pr, 'rx')
hold on
plot(Q_Or2, Pr, 'bx')
plot(x_Or2, y_Or2, 'b')
grid on; grid minor
title("Q, or. 1 and or. 2");  
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
axis([0 0.8 1 1.3]);
name = ["Or1, samples", "Or1, fitted curve", "Or2, samples", "Or2, fitted curve"];
legend(name, 'location', 'best')

end %compressorCurve

%% CORRECTION FACTORS
%Or. 2 correction factor, found with no factor on Or.1
corrFactOr2 = 1;
if corrFactOr2 == 1
filename = "ExData.xlsx";%   sheet = 1;  xlRange = "A3:D14";
sheet = readtable(filename, "Sheet", "OO2", 'Range', 'B3:V37');

[RPM, DV_pos, Q, Q_Or1, Q_Or2, Pr, m, m_Or1, m_Or2, P_bar, T_bar, Humid,...
    DP250_1, DP25_1, DPSmar_1, DP250_2, DP50_2, DP25_2, DPSmar_2]...
    =readDataTable(filename, "RO2", 'C3:U8', 1, 5); %Determines which cells to look at

funcName = "poly2"; %which degree the polynomial is. "poly2" = second degree.
[Q_fit, gof] = fit(Q, Pr, funcName);
[Q_Or1_fit, gof_Or1] = fit(Q_Or1, Pr, funcName);
[Q_Or2_fit, gof_Or2] = fit(Q_Or2, Pr, funcName);


x_Or1 = linspace(min(Q_Or1), max(Q_Or1), length(Q_Or1));
y_Or1 = x_Or1.^2 .* Q_Or1_fit.p1 + x_Or1 .* Q_Or1_fit.p2 + Q_Or1_fit.p3;
x_Or2 = linspace(min(Q_Or2), max(Q_Or2), 100);%length(Q_Or2));
y_Or2 = x_Or2.^2 .* Q_Or2_fit.p1 + x_Or2 .* Q_Or2_fit.p2 + Q_Or2_fit.p3;

%CORRECTION FACTORS
C_const = 0.029459958862728; %mean(Q_Or1 + Q_Or2)
C_lin = -1.046897203761607; %mean(Q_Or1 ./ Q_Or2);
C_cQuad = 0;        C_lQuad = 0;        C_quad = 0;

figure %Plotting Or 2 with correction factor CONSTANT
plot(Q_Or1_fit, Q_Or1, Pr, 'rx')
hold on
plot( C_const - Q_Or2, Pr, 'bx')
plot( C_const - x_Or2, y_Or2, 'b')
grid on; grid minor
tit = "Q, Or.2, correction factor, $C_{const} =$ " + C_const + ".";
title(tit);  
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
%axis([0.4 0.9 1 1.3]);
name = ["Or1, data", "Or1, fitted curve", ...
    "Or2, data, corrected", "Or2, fitted curve, corrected"];
legend(name, 'location', 'best')

figure %Plotting or 2 with correction factor LINEAR
plot(Q_Or1_fit, Q_Or1, Pr, 'rx')
hold on
plot(Q_Or2 * C_lin, Pr, 'bx')
plot(x_Or2 * C_lin, y_Or2, 'b')
grid on; grid minor
tit = "Q, Or.2, correction factor, $C_{lin} =$ " + C_lin + ".";
title(tit);  
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
%axis([0.4 0.9 1 1.3]);
name = ["Or1, data", "Or1, fitted curve", ...
    "Or2, data, corrected", "Or2, fitted curve, corrected"];
legend(name, 'location', 'best')

end %if corrFactOr2

%Or. 1 correction factor
corrFactOr1 = 0;
if corrFactOr1 == 1
figure
corrFact1 = 0.84522957261; %Or 1 correction factor
plot(Q_Or1 * corrFact1, Pr, 'r*')
hold on
plot(x_Or1 * corrFact1, y_Or1, 'r--')
plot(Q_Or1, Pr, 'rx')
plot(x_Or1, y_Or1, 'r')
plot(Q_Or2, Pr, 'bx')
plot(x_Or2, y_Or2, 'b')
grid on; grid minor
tit = "Q, Or.1, correction factor = " + corrFact1 + ".";
title(tit);  
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
%axis([0.4 0.9 1 1.3]);
name = ["Or1, corrected data", "Or1, fitted curve (corrected)", ...
    "Or1, raw data", "Or1, fitted curve (raw)", ...
    "Or2, data, corrected", "Or2, fitted curve, corrected"];
legend(name, 'location', 'best')
end %if corrFactOr1

correctionFactor_Mode = "forced";
switch correctionFactor_Mode
    case "forced"
        %In the "forced" mode, the correction factor for orifice 1 is a
        %vector and the correction factor used will be a linear
        %interpolation to get the closest alternative
        
        corrFactors = Q_Or2 ./ Q_Or1;

    otherwise
        return;
end%switch correctionFactor_Mode

%% Q vs Pr - plot, using "old" data (17.03, 25.03)
QvsPrplot = 0; %Determines whether or not figure below fires
if QvsPrplot == 1
figure
[RPM, DV_pos, Q, Q_Or1, Q_Or2, Pr, m, m_Or1, m_Or2, P_bar, T_bar, Humid,...
    DP250_1, DP25_1, DPSmar_1, DP250_2, DP50_2, DP25_2, DPSmar_2]...
    =readDataTable(filename, "OO2", 'B3:V37', 1, 12); %Determines which cells to look at

[Q_fit, gof] = fit(Q, Pr, funcName);
p = plot(Q_fit, Q, Pr, 'x');
%axis([0.2 1.8 1 1.3]);
hold on; grid on; grid minor
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
legName = ["Static samples", "Regression curve", "Predicted curve"];
legend(legName, 'location', 'best')

readFilename = "UelandQvsPrGraph.xlsx";
Tr = readtable(readFilename); [~, col] = size(Tr);
i = 0; %reusing this as an iteration variable
name = strings(1, col/3); %preallocating
    for n = 1:3:col
        i = i + 1;
        hold on
        Qr = Tr{:, n + 1};
        Pr_r = Tr{:, n + 2};
        name(i) = ( string(Tr{1, n}) + " " + string(Tr{2, n}) );
        if name(i) == "RPM 9021" %(name(i) == "DV-pos 44" || name(i) == "DV-pos 57" )%|| name(i) == "RPM 9021")
            %Ignores the DV-lines in the code
        else; plot(Qr, Pr_r, 'k');
        end %if
    end %for n = 1:3:col

    %Making the figure "pretty"
    xlabel('Volumetric flow, $Q$, $(m^3/s)$');
    ylabel('Pressure ratio, $Pr$, (-)');
    %title("Volumetric flow vs. Pressure ratio");%, Ueland data');
    hold on
    grid on
    grid minor
    axis([0.2 1.8 1 1.3]);
    legName = ["Static samples", "Regression curve", "Curves by Ueland"];
    legend(legName, 'location', 'best')

    %Overwriting legend-names with new ones:
    %name = [legName, "Thermodynamic equilibrium"];%"17.03, stable comp.curve", "25.03, low-flow comp.curve"];
    %legend(name, 'location', 'best');
end %if QvsPrplot == 1

%% Low-flow compressor curves in non-thermodynamic equilibrium


% figure(12)
% funcName = "poly2";
% [Q_fit_l, gof_l] = fit(Q, Pr_l, funcName);
% %plot(Q_fit_l, Q_l, Pr_l)
% %axis([0.2 1.8 1 1.3]);
% hold on; grid on; grid minor
% title("Volumetric flow vs Pressure ratio");  
% xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
% 
%     %Quadratic regression line (and function)
% x = linspace(Q_l(1), Q_l(end), length(Q_l));
% y = x.^2 .* Q_fit_l.p1 + x .* Q_fit_l.p2 + Q_fit_l.p3;
% plot(x, y)
% 
% leg = ["Exp. Data", "Quadratic regression line"];%funcName];
% legend(leg, 'location', 'best');

%% Orifice 2, sensor DP50
Or2DP50 = 1; %Determines whether or not the code in this chapter fires or not

if Or2DP50 == 1
readFilename = "ExData.xlsx";
sheet = readtable(readFilename, "Sheet", "RO2DP50", 'Range', 'A1:O2484'); %Reads data from reversed orifice 2 sheet

Q_pr1 = sheet.Q_pr1;
Pr_pr1 = sheet.Pr_pr1;
Q_pr2 = sheet.Q_pr2;
Pr_pr2 = sheet.Pr_pr2;
Q_Spr1 = sheet.Q_Spr1;
Pr_Spr1 = sheet.Pr_Spr1;
Q_Spr2 = sheet.Q_Spr2;
Pr_Spr2 = sheet.Pr_Spr2;

%sheet = readtable(filename, "Sheet", "OO2DP50"); %Reads data from original orifice 2 sheet
Q2_pr1 = sheet.Q2_pr1;
Pr2_pr1 = sheet.Pr2_pr1;
Q2_pr2 = sheet.Q2_pr2;
Pr2_pr2 = sheet.Pr2_pr2;
Q2_Spr1 = sheet.Q2_Spr1;
Pr2_Spr1 = sheet.Pr2_Spr1;

%Ensuring no cells contain NaN
Q_pr1 = Q_pr1(~isnan(Q_pr1));
Pr_pr1 = Pr_pr1(~isnan(Pr_pr1));
Q_pr2 = Q_pr2(~isnan(Q_pr2));
Pr_pr2 = Pr_pr2(~isnan(Pr_pr2));
Q_Spr1 = Q_Spr1(~isnan(Q_Spr1));
Pr_Spr1 = Pr_Spr1(~isnan(Pr_Spr1));
Q_Spr2 = Q_Spr2(~isnan(Q_Spr2));
Pr_Spr2 = Pr_Spr2(~isnan(Pr_Spr2));

Q2_pr1 = Q2_pr1(~isnan(Q2_pr1));
Pr2_pr1 = Pr2_pr1(~isnan(Pr2_pr1));
Q2_pr2 = Q2_pr2(~isnan(Q2_pr2));
Pr2_pr2 = Pr2_pr2(~isnan(Pr2_pr2));
Q2_Spr1 = Q2_Spr1(~isnan(Q2_Spr1));
Pr2_Spr1 = Pr2_Spr1(~isnan(Pr2_Spr1));

%Determining regression line, reversed orifice
funcName = "poly3"; %which degree the polynomial is. "poly2" = second degree.
%IF higher resolution is wanted, all x_pr and y_pr lines must be expanded
%to accomodate the new parameters of Q_fit.
[Q_fit_pr1, gof_pr1] = fit(Q_pr1, Pr_pr1, funcName);
[Q_fit_pr2, gof_pr2] = fit(Q_pr2, Pr_pr2, funcName);
[Q_fit_Spr1, gof_Spr1] = fit(Q_Spr1, Pr_Spr1, funcName);
[Q_fit_Spr2, gof_Spr2] = fit(Q_Spr2, Pr_Spr2, funcName);

[~, pos_pr1] = min(Q_pr1);      [~, pos2_pr1] = max(Q_pr1);
[~, pos_pr2] = min(Q_pr2);      [~, pos2_pr2] = max(Q_pr2);
[~, pos_Spr1] = min(Q_Spr1);      [~, pos2_Spr1] = max(Q_Spr1);
[~, pos_Spr2] = min(Q_Spr2);      [~, pos2_Spr2] = max(Q_Spr2);

x_pr1 = linspace(Q_pr1(pos_pr1), Q_pr1(pos2_pr1), 100);
y_pr1 = x_pr1.^3 .* Q_fit_pr1.p1 + x_pr1.^2 .* Q_fit_pr1.p2 + Q_fit_pr1.p3 .* x_pr1 + Q_fit_pr1.p4;
x_pr2 = linspace(Q_pr2(pos_pr2), Q_pr2(pos2_pr2), 100);
y_pr2 = x_pr2.^3 .* Q_fit_pr2.p1 + x_pr2.^2 .* Q_fit_pr2.p2 + Q_fit_pr2.p3 .* x_pr2 + Q_fit_pr2.p4;
x_Spr1 = linspace(Q_Spr1(pos_Spr1), Q_Spr1(pos2_Spr1), 100);
y_Spr1 = x_Spr1.^3 .* Q_fit_Spr1.p1 + x_Spr1.^2 .* Q_fit_Spr1.p2 + Q_fit_Spr1.p3 .* x_Spr1 + Q_fit_Spr1.p4;
x_Spr2 = linspace(Q_Spr2(pos_Spr2), Q_Spr2(pos2_Spr2), 100);
y_Spr2 = x_Spr2.^3 .* Q_fit_Spr2.p1 + x_Spr2.^2 .* Q_fit_Spr2.p2 + Q_fit_Spr2.p3 .* x_Spr2 + Q_fit_Spr2.p4;

%Determining regression line, original orifice (not reversed)
[Q2_fit_pr1, gof2_pr1] = fit(Q2_pr1, Pr2_pr1, funcName);
[Q2_fit_pr2, gof2_pr2] = fit(Q2_pr2, Pr2_pr2, funcName);
[Q2_fit_Spr1, gof2_Spr1] = fit(Q2_Spr1, Pr2_Spr1, funcName);

[~, pos_2_pr1] = min(Q2_pr1);      [~, pos2_2_pr1] = max(Q2_pr1);
[~, pos_2_pr2] = min(Q2_pr2);      [~, pos2_2_pr2] = max(Q2_pr2);
[~, pos_2_Spr1] = min(Q2_Spr1);      [~, pos2_2_Spr1] = max(Q2_Spr1);

x2_pr1 = linspace(Q2_pr1(pos_2_pr1), Q2_pr1(pos2_2_pr1), 100);
y2_pr1 = x2_pr1.^3 .* Q2_fit_pr1.p1 + x2_pr1.^2 .* Q2_fit_pr1.p2 + Q2_fit_pr1.p3 .* x2_pr1.^1 + Q2_fit_pr1.p4;
x2_pr2 = linspace(Q2_pr2(pos_2_pr2), Q2_pr2(pos2_2_pr2), 100);
y2_pr2 = x2_pr2.^3 .* Q2_fit_pr2.p1 + x2_pr2.^2 .* Q2_fit_pr2.p2 + Q2_fit_pr2.p3 .* x2_pr2.^1 + Q2_fit_pr2.p4;
x2_Spr1 = linspace(Q2_Spr1(pos_2_Spr1), Q2_Spr1(pos2_2_Spr1), 100);
y2_Spr1 = x2_Spr1.^3 .* Q2_fit_Spr1.p1 + x2_Spr1.^2 .* Q2_fit_Spr1.p2 + Q2_fit_Spr1.p3 .* x2_Spr1.^1 + Q2_fit_Spr1.p4;

%Plotting reversed orifice 2
f1 = figure; %plotting scatter plot for reversed orifice
p1 = plot(Q_pr1, Pr_pr1, 'x');
%axis([0.2 1.8 1 1.3]);
hold on; grid on; grid minor
p2 = plot(Q_pr2, Pr_pr2, 'x');
p3 = plot(Q_Spr1, Pr_Spr1, 'x');
p4 = plot(Q_Spr2, Pr_Spr2, 'x');
%set([p1], 'color', 'k')
%title("Q, mean");  
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
legName = ["Profile 1, samples", "Profile 2, samples", ...
    "Profile 1, S-volume, samples", "Profile 2, S-volume, samples" ...
    %"Profile 1, reg.curve", "Profile 2, reg.curve"...
    ];
legend(legName, 'location', 'best')

[RPM, DV_pos, Q, Q_Or1, Q_Or2, Pr, m, m_Or1, m_Or2, P_bar, T_bar, Humid,...
    DP250_1, DP25_1, DPSmar_1, DP250_2, DP50_2, DP25_2, DPSmar_2]...
    =readDataTable(filename, "RO2", 'B3:V8', 1, 5); %Determines which cells to look at


f2 = figure; %plotting regression curves for reversed orifice
p5 = plot(x_pr1, y_pr1);
hold on; grid on; grid minor
p6 = plot(x_pr2, y_pr2);
p7 = plot(x_Spr1, y_Spr1);
p8 = plot(x_Spr2, y_Spr2);
%set(p3, 'color', 'k');
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
legName = [...
    %"Profile 1, samples", "Profile 2, samples", ...
    "Profile 1, reg.curve", "Profile 2, reg.curve", ...
    "Profile 1, S-volume, reg.curve", "Profile 2, S-volume, reg.curve"...
    ];
legend(legName, 'location', 'best')

%Plotting original orifice (non-reversed)
f3 = figure; %plotting scatter plot for original orifice
p9 = plot(Q2_pr1, Pr2_pr1, 'x');
hold on; grid on; grid minor
p10 = plot(Q2_pr2, Pr2_pr2, 'x');
p11 = plot(Q2_Spr1, Pr2_Spr1, 'x');
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
legName = [ ...
    "Profile 1, samples", "Profile 2, samples", ...
    "Profile 1, S-volume, samples" ...
    ];
legend(legName, 'location', 'best');

f4 = figure; %plotting regression curves for reversed orifice
p12 = plot(x2_pr1, y2_pr1);
hold on; grid on; grid minor
p13 = plot(x2_pr2, y2_pr2);
p14 = plot(x2_Spr1, y2_Spr1);
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
legName = [ ...
    "Profile 1, reg.curve", "Profile 2, reg.curve", ...
    "Profile 1, S-volume, reg.curve" ...
    ];
legend(legName, 'location', 'best');
    %Quadratic regression line (and function)
% x_Or2 = linspace(Q_Or2(1), Q_Or2(end), length(Q_Or2));
% y_Or2 = x_Or2.^2 .* Q_Or2_fit.p1 + x_Or2 .* Q_Or2_fit.p2 + Q_Or2_fit.p3;
%plot(x, y)

f5 = figure;
p15 = plot(x_pr1, y_pr1);
hold on; grid on; grid minor
p16 = plot(x_pr2, y_pr2);
p17 = plot(x_Spr1, y_Spr1);
p18 = plot(x_Spr2, y_Spr2);
p19 = plot(x2_pr1, y2_pr1);
p20 = plot(x2_pr2, y2_pr2);
p21 = plot(x2_Spr1, y2_Spr1);
%set(p3, 'color', 'k');
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
legName = [...
    "Profile 1, reg.curve", "Profile 2, reg.curve", ...
    "Profile 1, S-volume, reg.curve", "Profile 2, S-volume, reg.curve"...
    "Profile 1, reg.curve", "Profile 2, reg.curve", ...
    "Profile 1, S-volume, reg.curve" ...
    ];
legend(legName, 'location', 'best')

set([p15, p19], 'color', 'b'); set([p16, p20], 'color', 'r'); 
set([p17, p18, p21], 'color', 'k');

figure%reversed scatter side-by-side
tiledlayout(1,2)
nexttile
p1 = plot(Q_pr1, Pr_pr1, 'x');
hold on; grid on; grid minor
p2 = plot(Q_pr2, Pr_pr2, 'x');
%set([p1], 'color', 'k')
%title("Q, mean");  
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
legName = ["Profile 1, samples", "Profile 2, samples", ...
    %"Profile 1, S-volume, samples", "Profile 2, S-volume, samples" ...
    %"Profile 1, reg.curve", "Profile 2, reg.curve"...
    ];
legend(legName, 'location', 'best')

nexttile
p3 = plot(Q_Spr1, Pr_Spr1, 'x', 'color', '#EDB120');
hold on; grid on; grid minor
p4 = plot(Q_Spr2, Pr_Spr2, 'x', 'color', '#7E2F8E');
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
legName = [...%"Profile 1, samples", "Profile 2, samples", ...
    "Profile 1, S-volume, samples", "Profile 2, S-volume, samples" ...
    %"Profile 1, reg.curve", "Profile 2, reg.curve"...
    ];
legend(legName, 'location', 'best')


end %Or2DP50


%% SV-profile
SVprofile = 1;
if SVprofile == 1
readFilename = "ExData.xlsx";
sheet = readtable(readFilename, "Sheet", "SVprofile", 'Range', 'A1:D35'); %Reads data from reversed orifice 2 sheet

Q = sheet.Q;    Q_Or1 = sheet.Q_Or1;    Q_Or2 = sheet.Q_Or2;   Pr = sheet.Pr;

startCell = 21; %determines where the first intervall starts
intervallCell1 = 29; %determines where the first intervall ends
intervallCell2 = 29; %determines where the second intervall starts
endCell = 34;%length(Q); %determines where the second intervall ends

%Determining regression line
funcName = "poly2"; %which degree the polynomial is. "poly2" = second degree.
[Q_fit_Or1, gof_Or1] = fit(Q_Or1(startCell:endCell), ...
    Pr(startCell:endCell), funcName);
[Q_fit_Or1_1, gof_Or1_1] = fit(Q_Or1(startCell:intervallCell1), ...
    Pr(startCell:intervallCell1), funcName);
[Q_fit_Or1_2, gof_Or1_2] = fit(Q_Or1(intervallCell2:endCell), ...
    Pr(intervallCell2:endCell), funcName);

[Q_fit_Or2, gof_Or2] = fit(Q_Or2(startCell:endCell),...
    Pr(startCell:endCell), funcName);
[Q_fit_Or2_1, gof_Or2_1] = fit(Q_Or2(startCell:intervallCell1), ...
    Pr(startCell:intervallCell1), funcName);
[Q_fit_Or2_2, gof_Or2_2] = fit(Q_Or2(intervallCell2:endCell), ...
    Pr(intervallCell2:endCell), funcName);

[Q_fit, gof_Q] = fit(Q(startCell:endCell), Pr(startCell:endCell), funcName);
[Q_fit_1, gof_Q_1] = fit(Q(startCell:intervallCell1), ...
    Pr(startCell:intervallCell1), funcName);
[Q_fit_2, gof_Q_2] = fit(Q(intervallCell2:endCell), ...
    Pr(intervallCell2:endCell), funcName);

[~, minPos_Or1] = min(Q_Or1);       [~, maxPos_Or1] = max(Q_Or1);
[~, minPos_Or2] = min(Q_Or2);       [~, maxPos_Or2] = max(Q_Or2);
[~, minPos_Q] = min(Q);             [~, maxPos_Q] = max(Q);

x_Or1 = linspace(Q_Or1(minPos_Or1), Q_Or1(maxPos_Or1), 100);
y_Or1 = x_Or1.^2 .* Q_fit_Or1.p1 + x_Or1 .* Q_fit_Or1.p2 + Q_fit_Or1.p3;
x_Or1_1 = linspace(Q_Or1(startCell), Q_Or1(intervallCell1), 100);
y_Or1_1 = x_Or1_1 .^ 2 .* Q_fit_Or1_1.p1 + x_Or1_1 .* Q_fit_Or1_1.p2 + ...
    Q_fit_Or1_1.p3;
x_Or1_2 = linspace(Q_Or1(intervallCell2), Q_Or1(endCell), 100);
y_Or1_2 = x_Or1_2 .^ 2 .* Q_fit_Or1_2.p1 + x_Or1_2 .* Q_fit_Or1_2.p2 + ...
    Q_fit_Or1_2.p3;

x_Or2 = linspace(Q_Or2(minPos_Or2), Q_Or2(maxPos_Or2), 100);
y_Or2 = x_Or2.^2 .* Q_fit_Or2.p1 + x_Or2 .* Q_fit_Or2.p2 + Q_fit_Or2.p3;
x_Or2_1 = linspace(Q_Or2(startCell), Q_Or2(intervallCell1), 100);
y_Or2_1 = x_Or2_1 .^ 2 .* Q_fit_Or2_1.p1 + x_Or2_1 .* Q_fit_Or2_1.p2 + ...
    Q_fit_Or2_1.p3;
x_Or2_2 = linspace(Q_Or2(intervallCell2), Q_Or2(endCell), 100);
y_Or2_2 = x_Or2_2 .^ 2 .* Q_fit_Or2_2.p1 + x_Or2_2 .* Q_fit_Or2_2.p2 + ...
    Q_fit_Or2_2.p3;

x_Q = linspace(Q(minPos_Q), Q(maxPos_Q), 100);
y_Q = x_Q.^2 .* Q_fit.p1 + x_Q .* Q_fit.p2 + Q_fit.p3;
x_Q_1 = linspace(Q(startCell), Q(intervallCell1), 100);
y_Q_1 = x_Q_1 .^ 2 .* Q_fit_1.p1 + x_Q_1 .* Q_fit_1.p2 + Q_fit_1.p3; %x_Q_1 .^ 3 .* Q_fit_1.p1 + x_Q_1.^2 .* Q_fit_1.p2 + x_Q_1 .* Q_fit_1.p3 + Q_fit_1.p4;
x_Q_2 = linspace(Q(intervallCell2), Q(endCell), 100);
y_Q_2 = x_Q_2 .^ 2 .* Q_fit_2.p1 + x_Q_2 .* Q_fit_2.p2 + ...
    Q_fit_2.p3;

if Or2DP50 == 1 %adding "regular" compressor curves to other 2 curves
    %Copy-paste to get 2 figures (so the "regular" compressor curve can be
    %added with these two; "reduced" flow, and "reversed" flow)
    corrNumb = 0;%0.0075;
    C_const = 0;%0.029459958862728; % 0; %
    tweak = 0.006;

    %Plotting reversed orifice 2
    f1 = figure; %plotting scatter plot for reversed orifice
    p1 = plot(Q_pr1, Pr_pr1 + C_const, 'x');
    %axis([0.2 1.8 1 1.3]);
    hold on; grid on; grid minor
    p2 = plot(Q_pr2, Pr_pr2 + C_const, 'x');
    p3 = plot(Q_Spr1, Pr_Spr1 + C_const, 'x');
    p4 = plot(Q_Spr2, Pr_Spr2 + C_const, 'x');
    %set([p1], 'color', 'k')
    %title("Q, mean");  
    xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
    legName = ["Profile 1, samples", "Profile 2, samples", ...
        "Profile 1, S-volume, samples", "Profile 2, S-volume, samples" ...
        %"Profile 1, reg.curve", "Profile 2, reg.curve"...
        ];
    legend(legName, 'location', 'best')

    f6 = figure; %compressor curves
    p22 = plot(x_pr1, y_pr1 + C_const);
    hold on; grid on; grid minor
    p24 = plot(x_Spr1, y_Spr1 + C_const);
    p29 = plot(Q(startCell:intervallCell1), Pr(startCell:intervallCell1) + tweak, ...
        'x', 'color', '#77AC30');
    p30 = plot(x_Q_1, y_Q_1 + tweak, 'color', '#77AC30');
    p25 = plot(x_Spr2, y_Spr2 + C_const);
    p26 = plot(x2_pr1, y2_pr1 - corrNumb);
    p23 = plot(x_pr2, y_pr2 + C_const);
    p27 = plot(x2_pr2, y2_pr2 - corrNumb);
    p28 = plot(x2_Spr1, y2_Spr1 - corrNumb);
    %set(p3, 'color', 'k');
    xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
    legName = [...
        "Profile 1, reg.curve", "Profile 2, reg.curve", ...
        "Profile 1, S-volume, reg.curve", "Profile 2, S-volume, reg.curve"...
        "Profile 1, reg.curve", "Profile 2, reg.curve", ...
        "Profile 1, S-volume, reg.curve" ...
        ];
    legend(legName, 'location', 'best')
    
    set([p22, p26], 'color', 'b'); set([p23, p27], 'color', 'b'); 
    set([p24, p25, p28], 'color', 'k'); set([p29, p30], 'color', 'r')

    legName = [...
        "Regular profile, reg.curve", ... %"Regular Profile 2, reg.curve", ...
        "S-volume profile, reg.curve", ...
        "S-volume, Static samples", ...
        "Static samples reg.curve", ...
        %"Profile 1, S-volume, reg.curve", "Profile 2, S-volume, reg.curve"...
        %"Profile 1, reg.curve", "Profile 2, reg.curve", ...
        %"Profile 1, S-volume, reg.curve", ...
        %"Samples, static, S-volume", "Reg.curve, static, S-volume" ...
        ];
    legend(legName, 'location', 'best')
end %Or2DP50

f1 = figure;
p1 = plot(Q_Or1(startCell:endCell), Pr(startCell:endCell), 'bx');
hold on; grid on; grid minor;
p2 = plot(Q_Or2(startCell:endCell), Pr(startCell:endCell), 'rx');
p3 = plot(Q(startCell:endCell), Pr(startCell:endCell), 'kx');
p4 = plot(x_Or1, y_Or1, 'b');
p5 = plot(x_Or2, y_Or2, 'r');
p6 = plot(x_Q, y_Q, 'k');
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");
legName = ["Orifice 1, samples", "Orifice 2, samples", "Average, samples",...
    "Orifice 1, reg.curve", "Orifice 2, reg.curve", "Average, reg.curve"];
legend(legName, 'location', 'best')

f2 = figure;
p7 = plot(Q_Or1(startCell:endCell), Pr(startCell:endCell), 'bx');
hold on; grid on; grid minor;
p8 = plot(Q_Or2(startCell:endCell), Pr(startCell:endCell), 'rx');
p9 = plot(Q(startCell:endCell), Pr(startCell:endCell), 'kx');
p10 = plot(x_Or1_1, y_Or1_1, 'b');
p12 = plot(x_Or2_1, y_Or2_1, 'r');
p14 = plot(x_Q_1, y_Q_1, 'k');
p11 = plot(x_Or1_2, y_Or1_2, 'b');
p13 = plot(x_Or2_2, y_Or2_2, 'r');
p15 = plot(x_Q_2, y_Q_2, 'k');
xlabel("Volumetric flow, Q, ($m^3/s$)"); ylabel("Pressure ratio, Pr, (-)");

legName = ["Orifice 1, samples", "Orifice 2, samples", "Average, samples",...
    "Orifice 1, reg.curve", ... "Orifice 1, reg.curve 2", ...
    "Orifice 2, reg.curve", ... "Orifice 2, reg.curve 2", ...
    "Average, reg.curve 1", ... "Average, reg.curve 2" ...
    ];
legend(legName, 'location', 'best')

end %SVprofile

%% Functions
function [RPM, DVpos, Q, Qor1, Qor2, Pr, m, mor1, mor2, pbar, Tbar, ...
    hum, DP250_1, DP25_1, DPSmar_1, DP250_2, DP50_2, DP25_2, DPSmar_2] ...
    = readDataTable (readFilename, sheetName, range, fromCell, toCell)
%This function will attempt to read the data values given in a document
%like ExData. NOT VERY COMPATIBLE
sheet = readtable(readFilename, "Sheet", sheetName, 'range', range);

RPM = sheet.RPM(fromCell:toCell);
DVpos = sheet.DV_pos(fromCell:toCell);
Q = sheet.Q(fromCell:toCell);
Qor1 = sheet.Q_Or1(fromCell:toCell);
Qor2 = sheet.Q_Or2(fromCell:toCell);
Pr = sheet.Pr(fromCell:toCell);
m = sheet.m(fromCell:toCell);
mor1 = sheet.m_Or1(fromCell:toCell);
mor2 = sheet.m_Or2(fromCell:toCell);
    %Atmospheric conditions
pbar = sheet.P_bar(fromCell:toCell); %P barometric, bara
Tbar = sheet.T_bar(fromCell:toCell); %T barometric, celsius
hum = sheet.Humid(fromCell:toCell); % Barometric humidity, (%)
    %DP pressures _1 = or1, _2 = or2. Unit is mbar(g)
DP250_1 = sheet.DP250_1(fromCell:toCell);
DP25_1 = sheet.DP25_1(fromCell:toCell);
DPSmar_1 = sheet.DPSmar_1(fromCell:toCell);
DP250_2 = sheet.DP250_2(fromCell:toCell);
DP50_2 = sheet.DP50_2(fromCell:toCell);
DP25_2 = sheet.DP25_2(fromCell:toCell);
DPSmar_2 = sheet.DPSmar_2(fromCell:toCell);


end %function readDataTable


