%% About
%Script created by Sander K. Arstad in Autumn of 2021, in an attempt to make .tdms
%files more accessible. "v2p6"-files necessary to run the code (borrowed
%from Jim Hokanson (2021). TDMS Reader 
%(https://www.mathworks.com/matlabcentral/fileexchange/30023-tdms-reader), 
%MATLAB Central File Exchange. Retrieved Late October, 2021.). User MUST 
%add v2p6-files to path (specifically, the "v2p6" folder and
%"v2p6\tdmsSubfunctions" folder) for this script to function properly.

%Generally, from Erik Langørgen: MUST verify data, and not use anything 
%uncritically. Dismiss "crazy values" (F500_0's T_16 gets 0 deg. C for one 
%value, as an example)
%Units are ALWAYS in SI-units (pressure might be given as bar or mbar,
%absolute-value (no negative pressures)

%% Function input -- No longer in use
%function NTNUExperimentalReader (fileName) %fileName MUST be a string that 
%ends in .tdms
%% Aquiring file
addpath("C:\Users\sanderka\OneDrive\NTNU\Masteroppgave\Matlab\v2p6")
addpath("C:\Users\sanderka\OneDrive\NTNU\Masteroppgave\Matlab\v2p6\tdmsSubfunctions")

%filename = fileName;
%Change the filename to access a new data-test
path = "C:\Users\sanderka\OneDrive\NTNU\Masteroppgave\Matlab\Analyzed data\ExperimentalData\";
folder = "0504";
filename = "Slow_9000_profile_SVolume.tdms"; %'Slow_standstill test_0803.tdms'; %'Slow_6000-9000-11000_test.tdms';
testName = "low-profile"; %The name of this test. Keep it short

%The two values below must be updated whenever the filename is changed. Set
%the value 0 if the parameter is transient. If a parameter is a steady
%state parameter, write what the steady state value of said parameter is.
rpm = 0; % the steady-state RPM of this experiment.
dv = 0; % the steady-state dv-pos of this experiment.

experimental = 1; %Set to 1 if experimental data is to be analysed
simMatch = 0;   %If there is a matching simulation in the TransientFilename, set 1, else
%set 0.

%Use "clean" temperatures (if available)? 1 for yes, 0 for no (use raw)
cTemps = 1;

%Save the Q vs Pr plot for this experiment?
saveQPrData = 0; % if = 1, saves the current Q vs p2/p1- data to the excel sheet "QvsPrGraph"
saveQPrDataSheet = "Orifice"; %The data sheet where the data will be saved. 
	%Some pre-made options available: "GeneralAverage", for an average of
	%all sensors; "Orifice", for an average in all orifices; "Sensors", for
	%saving individual sensors

addpath(path + folder);
filename = char(filename);
data = TDMS_getStruct(filename);
%The excel-sheet with transient data recorded.
TransientFilename = 'oldTransientData.xlsx';
%TransientData = xlsread(TransientFilename);

%% Selecting study
fprintf("Data:\n");
fprintf("D for DAQ_0 (W to add TempWaterFT1_5)\t\t Temperatures " + ...
    "throughout system\nF for F500_0\t\t\t\t Multiple " + ...
    "temperature-sensors at compressor inlet and outlet\nP for " + ...
    "PXI_0 \t\t\t\t Multiple pressure-sensors throughout system\n" + ...
    "T for PTU307_0 \t\t\t\t Atmospheric values\n" + ...
    "R for cRIO_0 \t\t\t\t Valve positions, with setpoint.\n" + ...
    "C for CANopen_0 \t\t\t Compressor torque\n" + ...
    "A for all" + ...
    "\nAny other character for continue (NOT simply 'enter')" + ...
    "\n\nLower case also allowed\n\n\t\t");
inp = input('Input: ', 's');

%%% Specifying what type of experiment it is
if dv ~= 0 && rpm == 0 %if rpm is transient
    RPMTransient = 1; %If Transient behaviour when changing the DV is
            %being studied, set this to 1, else, 0
    DVTransient = 0;%If Transient behaviour when changing the comp. rpm is
            %being studied, set this to 1, else, 0
elseif rpm ~= 0 && dv == 0 %if dv is transient
    RPMTransient = 0;
    DVTransient = 1;
elseif rpm ~= 0 && dv ~= 0 %if neither is transient
    RPMTransient = 0; 
    DVTransient = 0;
elseif rpm == 0 && dv == 0 %if both are transient
    RPMTransient = 1;
    DVTransient = 1;
else
    error('Logical error. Should not be possible.');
end %if rpm ~= 0, dv ~=0

if (rpm == 0 && dv == 0 && saveQPrData ~= 0)
    fprintf("An experiment where both the RPM and DV-position \n" + ...
        "changes throughout the course of the experiment has \n" +...
        "been detected.\n" + ...
        "Please write the name you would like to save the \n" +...
        "Q vs. P2/P1 data under: \n" +...
        "(only a combination of letters valid)\n");
    saveQPrDataName = input(" ", "s"); %this is used in the script "TransentResponse"
end %if saveQPrData ...

%% Data extraction and selectable plots
loggingFreq = 10; % Hz, the logging frequency of the PXI sensor
experimentDuration = length(data.PXI_0.Time.data)/loggingFreq; %s, recording frequency of 2 Hz.

%tempTol is used to determine what amount of temperature-deviation is
%accepted from one temperature measurement to the next. "Raw" data values
%are always kept, but if used, a "clean" data value is also given
tempTol = 7 ; %deg C, a greater variation than tempTol deg C gets removed


%%% DAQ_0
%DAQ_0
    %Time
    %TempInletA
    %TempWaterInj1
    %Orifice
    %TempWater_FT1_5
daqTime = data.DAQ_0.Time.data; %raw time-stamps
daqTime = (daqTime - daqTime(1)); %making sure it starts at 0
daqTime = daqTime * (experimentDuration /daqTime(end)); %s, scaling the time-scale to be in seconds
%^ this procedure was repeated for all time-stamps.


TempInletA = data.DAQ_0.TempInlet_A.data; % deg C, temp right before IGVs
TempWaterInj1 = data.DAQ_0.TempWaterInj1.data; % deg C, temp right before water injection (ca. 3 m below orifice 1)
TempOrifice1 = data.DAQ_0.Orifice.data; % deg C, temp over 1st orifice  % TempOrifice1 = data.DAQ_0.TempOrifice.data; %
TempOrifice2 = TempOrifice1; %TempOrifice2 = data.DAQ_0.Orifice.data; %deg C, temp over orifice
TempWaterFT1_5 = data.DAQ_0.TempWater_FT1_5.data; % deg C, unsure, another water injection?

[cleanTempOrifice, cOrificeTimes] = cleanTemp(TempOrifice1, tempTol);

if ((inp == 'D') || (inp == 'W')) || ((inp == 'd') || inp == 'w') || ...
        (inp == 'a' || inp == 'A')
    figure
    tempPlotTime(TempInletA, 'TempInletA', daqTime);
    
    figure
    tempPlotTime(TempWaterInj1, 'TempWaterInj1', daqTime);
    
    figure
    tempPlotTime(TempOrifice1, 'Temperature first Orifice', daqTime);
    
    if cOrificeTimes > 0
        figure
        tempPlotTime(cleanTempOrifice, 'Cleaned TempOrifice', daqTime);
        fprintf('\n%.f data points were changed for inlet temperatures.\n', cOrificeTimes);
    end %if cOrificeTimes > 0
    
    figure
    plot(daqTime, TempInletA)
    hold on
    plot(daqTime, TempWaterInj1)
    plot(daqTime, TempOrifice1)
    if (inp == 'w')
        plot(daqTime, TempWaterFT1_5)
        leg = ["Temp Inlet A"; "Temp Water Injection 1"; "Temp Orifice 1";...
            "Temp Water FT1_5"];
        figure
        tempPlotTime(TempWaterFT1_5, 'TempWaterFT1 5', daqTime);
    else
        leg = ["Temp Inlet A"; "Temp Water Injection 1"; "Temp Orifice 1"];
    end %if inp == 'DF'
    xlabel("Time, $s$");
    ylabel("Temperature, $^\circ C$");
    title("Combined DAQ-temperature plot");
    grid on
    grid minor
    legend(leg, 'location', 'best');

    
end %if inp == D || inp == 'DF'



%%% F500_0
%F500_0
    %Time
    %4 first for input
    %4 next for output

%Validation needed
F500Time = data.F500_0.Time.data;
F500Time = (F500Time - F500Time(1));
F500Time = F500Time * (experimentDuration / F500Time(end));

TempInlet16 = data.F500_0.TT_500_16.data; % deg C, temp inlet sensor 16
TempInlet17 = data.F500_0.TT_500_17.data; % deg C, temp inlet sensor 17
TempInlet18 = data.F500_0.TT_500_18.data; % deg C, temp inlet sensor 18
TempInlet19 = data.F500_0.TT_500_19.data; % deg C, temp inlet sensor 19

TempOutlet20 = data.F500_0.TT_500_20.data; % deg C, temp outlet sensor 20
TempOutlet21 = data.F500_0.TT_500_21.data; % deg C, temp outlet sensor 21
TempOutlet22 = data.F500_0.TT_500_22.data; % deg C, temp outlet sensor 22
TempOutlet23 = data.F500_0.TT_500_23.data; % deg C, temp outlet sensor 23

[cleanTempInlet16, c16Times] = cleanTemp(TempInlet16, tempTol); % deg C, clean temp inlet sensor 16
[cleanTempInlet17, c17Times] = cleanTemp(TempInlet17, tempTol); % deg C, clean temp inlet sensor 17
[cleanTempInlet18, c18Times] = cleanTemp(TempInlet18, tempTol); % deg C, clean temp inlet sensor 18
[cleanTempInlet19, c19Times] = cleanTemp(TempInlet19, tempTol); % deg C, clean temp inlet sensor 19

[cleanTempOutlet20, c20Times] = cleanTemp(TempOutlet20, tempTol); % deg C, clean temp outlet sensor 20
[cleanTempOutlet21, c21Times] = cleanTemp(TempOutlet21, tempTol); % deg C, clean temp outlet sensor 21
[cleanTempOutlet22, c22Times] = cleanTemp(TempOutlet22, tempTol); % deg C, clean temp outlet sensor 22
[cleanTempOutlet23, c23Times] = cleanTemp(TempOutlet23, tempTol); % deg C, clean temp outlet sensor 23


if (inp == 'F') || (inp == 'f') || (inp == 'A' || inp == 'a')
    %raw values
    figure
    tempPlotTime(TempInlet16, '', F500Time); % a bit faulty values
    hold on;
    tempPlotTime(TempInlet17, '', F500Time);
    hold on;
    tempPlotTime(TempInlet18, '', F500Time);
    hold on;
    tempPlotTime(TempInlet19, 'Raw Temp. Inlet 16 - 19', F500Time);
    legend(['TempInlet16'; 'TempInlet17'; 'TempInlet18'; 'TempInlet19'], 'location', 'best');
    %legend(['TempInlet17'; 'TempInlet18'; 'TempInlet19'], 'location', 'best');
        
    figure
    tempPlotTime(TempOutlet20, '', F500Time); %a bit faulty values
    hold on;
    tempPlotTime(TempOutlet21, '', F500Time); 
    hold on;
    tempPlotTime(TempOutlet22, '', F500Time);
    hold on;
    tempPlotTime(TempOutlet23, 'Raw Temp. Outlet 20 - 23', F500Time);
    legend(['TempOutlet20'; 'TempOutlet21'; 'TempOutlet22'; 'TempOutlet23'], 'location', 'best');
    %legend(['TempOutlet21'; 'TempOutlet22'; 'TempOutlet23'], 'location', 'best');
    
    %if cleaning was done
    cInlet = c16Times + c17Times + c18Times + c19Times;
    cOutlet = c20Times + c21Times + c22Times + c23Times;
    if cInlet > 0
        fprintf('\n%.f data points were changed for inlet temperatures.\n', cInlet);
        %cleaned data values
        figure
        tempPlotTime(cleanTempInlet16, '', F500Time); % a bit faulty values
        hold on;
        tempPlotTime(cleanTempInlet17, '', F500Time);
        hold on;
        tempPlotTime(cleanTempInlet18, '', F500Time);
        hold on;
        tempPlotTime(cleanTempInlet19, 'Clean Temp. Inlet 16 - 19', F500Time);
        legend(['TempInlet16'; 'TempInlet17'; 'TempInlet18'; 'TempInlet19'], 'location', 'best');
        %legend(['TempInlet17'; 'TempInlet18'; 'TempInlet19'], 'location', 'best');
    end % if cInlet > 0
    if cOutlet > 0
        fprintf('\n%.f data points were changed for outlet temperatures.\n', cOutlet);
        %cleaned data values
        figure
        tempPlotTime(cleanTempOutlet20, '', F500Time); %a bit faulty values
        hold on;
        tempPlotTime(cleanTempOutlet21, '', F500Time); 
        hold on;
        tempPlotTime(cleanTempOutlet22, '', F500Time);
        hold on;
        tempPlotTime(cleanTempOutlet23, 'Clean Temp. Outlet 20 - 23', F500Time);
        legend(['TempOutlet20'; 'TempOutlet21'; 'TempOutlet22'; 'TempOutlet23'], 'location', 'best');
    end % Outlet > 0
    
    
end %if inp == F



%%% PXI_0
%PXI_0 "The main data-logger". Communicates to DAQ
    % time = ca. 5 min
    %dev_ai5 = device analog inngang 5 (ikke i bruk)
    %generell regel; alt med navn er i bruk. Eksemplet over er ikke i bruk
    %PE_V1-2 = dynamic venturi pressure
    %Dp2_Orifice = Orifice pressure differential. 
        %Measures pressure diff. over orifice
    %'Torque from VSD' not in use

%Validation needed
pxiTime = data.PXI_0.Time.data;
pxiTime = (pxiTime - pxiTime(1));
pxiTime = pxiTime * (experimentDuration / pxiTime(end));

%Compressor
PressureInletA = data.PXI_0.Pressure_Inlet_A.data; % mbar, pressure inlet sensor A
PressureInletB = data.PXI_0.Pressure_Inlet_B.data;
PressureInletC = data.PXI_0.Pressure_Inlet_C.data;
PressureInletD = data.PXI_0.Pressure_Inlet_D.data;
PressureOutletA = data.PXI_0.Pressure_Outlet_A.data;
PressureOutletB = data.PXI_0.Pressure_Outlet_B.data;
PressureOutletC = data.PXI_0.Pressure_Outlet_C.data;
PressureOutletD = data.PXI_0.Pressure_Outlet_D.data;

%Orifice1
Orifice1InletPressure = data.PXI_0.PT_3_3.data;
Orifice1Smar = data.PXI_0.DP_Smar_Orifice1.data; %[-50, 100] mbar % %Orifice1Smar = data.PXI_0.DP_Smar_Orifice.data; %
Orifice1DP250 = data.PXI_0.DP1_Orifice1.data;   %+- 250 mbar
Orifice1DP25 = data.PXI_0.DP2_Orifice1.data;    %+- 25 mbar

%Orifice2
Orifice2InletPressure = data.PXI_0.P1_Orifice2.data;
Orifice2DP250 = data.PXI_0.DP1_Orifice2.data;   %+- 250 mbar
Orifice2DP50 = data.PXI_0.DP2_Orifice2.data;    %+- 50 mbar
Orifice2DP25 = data.PXI_0.DP3_Orifice2.data;    %+- 25 mbar
Orifice2Smar = data.PXI_0.DP_Smar_Orifice2.data;%[-50, 250] mbar
%Bull nose, or Or. 2 (depending on experiment)
DP10 = data.PXI_0.DP_10mBar.data;               %+- 10 mbar

if (inp == 'P') || (inp == 'p') || (inp == 'A' || inp == 'a')
    figure
    plotTime(PressureInletA, '','', pxiTime);
    hold on
    plotTime(PressureInletB, '','', pxiTime);
    hold on
    plotTime(PressureInletC, '','', pxiTime);
    hold on
    plotTime(PressureInletD, 'Pressure, mbar', ...
        'Compressor inlet pressure', pxiTime);
    legend(['Pressure Inlet A'; 'Pressure Inlet B'; 'Pressure Inlet C';...
        'Pressure Inlet D'], 'location', 'best');
    
    figure
    plotTime(PressureOutletA, '','', pxiTime);
    hold on
    plotTime(PressureOutletB, '','', pxiTime);
    hold on
    plotTime(PressureOutletC, '','', pxiTime);
    hold on
    plotTime(PressureOutletD, 'Pressure, mbar', ...
        'Compressor outlet pressure', pxiTime);
    legend(['Pressure Inlet A'; 'Pressure Inlet B'; 'Pressure Inlet C';...
        'Pressure Inlet D'], 'location', 'best');
    
    %First orifice
    figure
    tiledlayout(3, 1)
    %first tile
    nexttile
    plotTime(Orifice1InletPressure, '', '', pxiTime);
    hold on
    plotTime(Orifice1InletPressure - Orifice1DP25, '', '', pxiTime);
    hold on
    plotTime(Orifice1InletPressure - Orifice1DP250, 'Pressure, mbar', ...
        'Pressure difference over first orifice', pxiTime);
    legend(['P1                '; 'P2, 25mbar sensor '; 'P2, 250mbar sensor'], ...
        'location', 'best');
    %second tile
    nexttile
    plotTime(Orifice1DP25, 'Pressure, mbar', ...
        'DP, recorded by 25mbar sensor', pxiTime);
    %third tile
    nexttile
    plotTime((Orifice1DP250), 'Pressure, mbar', ...
        'DP, recorded by 250mbar sensor', pxiTime);
    
    %Second orifice
    figure
    tiledlayout(3, 1)
    %first tile
    nexttile
    plotTime(Orifice2InletPressure, '', '', pxiTime);
    hold on
    plotTime(Orifice2InletPressure - Orifice2DP50, '', '', pxiTime);
    hold on
    plotTime(Orifice2InletPressure - Orifice2DP250, 'Pressure, mbar', ...
        'Pressure difference over second orifice', pxiTime);
    legend(['P1                '; 'P2, 25mbar sensor '; 'P2, 250mbar sensor'], ...
        'location', 'best');
    %second tile
    nexttile
    plotTime(Orifice2DP50, 'Pressure, mbar', ...
        'DP, recorded by 25mbar sensor', pxiTime);
    %third tile
    nexttile
    plotTime((Orifice2DP250), 'Pressure, mbar', ...
        'DP, recorded by 250mbar sensor', pxiTime);
    
end %if inp == p


%%% PRU307_0
%PTU307_0 Barometric station at inlet of rig (insug fra rom-luften)
    %Measures atmospheric condition
    %XT_3_1P = pressure
    %XT_3_1T = temp
    %XT_3_1R = relative air humidity

%Validation needed
ptuTime = data.PTU307_0.Time.data;
ptuTime = (ptuTime - ptuTime(1));
ptuTime = ptuTime * (experimentDuration / ptuTime(end));

XT3_1P = data.PTU307_0.XT_3_1P.data * 10^-5; % bar, atmospheric pressure
XT3_1T = data.PTU307_0.XT_3_1T.data; % deg C, atmospheric temperature
XT3_1R = data.PTU307_0.XT_3_1R.data; % %, relative air humidity

if (inp == 'T') || (inp == 't') || (inp == 'A' || inp == 'a')
    figure
    tempPlotTime(XT3_1T, 'Atmospheric temperature', ptuTime);
    
    figure
    plotTime(XT3_1P, 'Pressure, bar', 'Atmospheric pressure', ptuTime);
    
    figure
    plotTime(XT3_1R, 'Humidity, \%', 'Relative air humidity', ptuTime);
end % if inp == T



%%% cRIO_0
%cRIO_0 Valve control for water valve
    %outlet ca 60 % (open. 100 % = completly open)
    %everything is valid values.

%Validation needed
crioTime = data.cRIO_0.Time.data;
crioTime = (crioTime - crioTime(1));
crioTime = crioTime * (experimentDuration / crioTime(end));

AntiSurge = data.cRIO_0.AntiSurge.data;
AntiSurgeSetpoint = data.cRIO_0.AntiSurge_Setpoint.data;
Outlet = data.cRIO_0.Outlet.data;
OutletSetpoint = data.cRIO_0.Outlet_Setpoint.data;
Water = data.cRIO_0.Water.data;
WaterSetpoint = data.cRIO_0.Water_Setpoint.data;
OutletPressureCRIO = data.cRIO_0.OutletPressureCRIO.data;

if (inp == 'R') || (inp == 'r') || (inp == 'A' || inp == 'a')
    figure
    plotTime(AntiSurge,'','', crioTime);
    hold on;
    plotTime(AntiSurgeSetpoint, 'Anti-Surge valve pos', 'Anti-Surge valve position. 100 = Open', crioTime);
    legend(['Recorded position'; 'Setpoint         '], 'location', 'best');
    
    figure
    plotTime(Outlet,'','', crioTime);
    hold on;
    plotTime(OutletSetpoint, 'Outlet valve pos', 'Outlet valve position. 100 = Open', crioTime);
    legend(['Recorded position'; 'Setpoint         '], 'location', 'best');
    
    figure
    plotTime(Water,'','', crioTime)
    hold on;
    plotTime(WaterSetpoint, 'Water injection valve position', ...
        'Valve position', crioTime);
    legend(['Recorded position'; 'Setpoint         '], 'location', 'best');
end %if inp == r


%%% CANopen_0
%CANopen_0 records the shaft torque through two sensors, LP1 and LP2
%(Low-Pass 1 and 2). Data is in Nm

canTime = data.CANopen_0.Time.data;
canTime = (canTime - canTime(1));
canTime = canTime * (experimentDuration / canTime(end));

shaftTorque1 = data.CANopen_0.LP1.data;
shaftTorque2 = data.CANopen_0.LP2.data;

if (inp == 'C' || inp == 'c') || (inp == 'A' || inp == 'a')
    figure
    plot(canTime, shaftTorque1)
    hold on
    plotTime(shaftTorque2, 'Shaft torque, Nm', 'Compressor Shaft Torque',...
        canTime)
    leg = ["LP1"; "LP2"];
    legend(leg, 'location', ' best');
end %if inp == c

%% Transient Analysis
%Extracting data from TransientData. Needs to be on the following format:
    %Time (s), Compressor speed (rpm), DV-pos (%), p1 (comp inlet press, 
    %mbar), T1 (comp inlet temp, C), p2 (mbar), T2 (mbar), 
    %Head (poly, kJ/gh), Efficiency (poly, %), mass flow (kg/s), 
    %Volumetric flow (m3/s), power (compressor, kW)
[simTime, simRPM, simDVpos, sim_p1, simT1, sim_p2, simT2, simHead, ...
    simEta, sim_m, simQ, simP] ...
    = dataExtracter('TransientData.xlsx', 'A', 'Exp. 5');

%%%% Some general plots. Controlled in "selecting study"-part
% Compressor speed
shaftSpeedRPM = data.PXI_0.Shaft_Speed.data; %rpm
%shaftSpeed = shaftSpeedRPM * 2 * pi/60; % rad/s

%pxiTime is the relevant timing here
leg = ""; %preallocating. Only used in if below; is (probably) overwritten
if (RPMTransient) %Plotting compressor speed development, 
    %if transient RPM is selected
    figure
    if simMatch
        %Adds compressor speeds from DV-simulation in RPM, if simData
        %available
        plot(simTime, simRPM, '-.');
        hold on
        %plot(time, rpm, ':');
        leg = ["Simulation Data"];
    end %if DVstudy and speedstudy and simMatch 
    if leg == "" %Ensures correct legend
        leg = "Experimental Data";
    else
        leg(end + 1) = "Experimental Data";
    end %leg == ""
    plot(pxiTime, shaftSpeedRPM, '-')
    xlabel('Time, $s$');
    ylabel('Compressor speed, RPM')
	grid on
    title('Compressor speed');
    legend(leg', 'location', 'best')
end % if speedTime ==1

%DV development
DVpos = data.cRIO_0.Outlet.data;
DVsetpoint = data.cRIO_0.Outlet_Setpoint.data;
%crioTime is the relevant timing here.
leg = ""; %preallocating. Only used in if below; is (probably) overwritten
if DVTransient %Plotting DV-pos development
    figure
    if simMatch %Adds DV simulation plots if DV is being studied
        plot(simTime, simDVpos, '-.')
        hold on
        leg = ["Simulation Data"];
    end %if DVstudy
    if leg == "" %Ensures correct legend
        leg(1) = "Actual DV-pos.";
        leg(end + 1) = "DV-setpoint";
    else
        leg(end + 1) = "Actual DV-pos.";
        leg(end + 1) = "DV-setpoint";
    end %leg == ""
    plot(crioTime, DVpos);
    hold on
    plot(crioTime, DVsetpoint, '--') 
    xlabel('Time, $s$');
    ylabel('$\%$ Open');
    title('Discharge valve (DV) position');
    grid on
    grid minor
    legend(leg', 'location', 'best')
end % if DVdev == 1


%%% Experimenting (OUTDATED)


%    pxiTime(length(pxiTime)+1) = pxiTime(length(pxiTime));

%    matrix = [daqTime ; pxiTime];


%Calculated_0:
    %Time
    %Qair
    %GMF
    

%% Main-function end
%end %NTNUExperimentalReader 
%% Functions. Not all are in use 
function tempPlotTime (tempData, Title, time)
    plot(time, tempData)
    title(Title);
    xlabel('Time, s');
    ylabel('Temp, $^\circ C$');
    grid on
    grid minor
end %func tempPlotTime
function plotTime (yData, yLabel, Title, time)
    plot(time, yData)
    title(Title);
    xlabel('Time, s');
    ylabel(yLabel);
    grid on
    grid minor
end %func plotTime
function [cTemp, cleaned] = cleanTemp (Temp, tempTol)
    %This function helps clean up some of the wierd temperature readings
    cleaned = 0; %recounts how many times information was changed
    for i = 2:length(Temp)
        if (abs(Temp(i) - Temp(i-1)) > tempTol) %if the next temperature-data varies too much, it is removed
            Temp(i) = Temp(i-1);
            cleaned = cleaned + 1;
        end %if deltaTemp > tempTol
    end%for length(Temp)
    cTemp = Temp;
end %func cleanTemp
function [Time, CompSpeedRPM, DVpos, p1, T1, p2, T2, polyHead, etaP, ...
    m, Q, pwr] = dataExtracter(filename, fromColumn, sheetName)
%Extracts the wanted data from the column fromColumn to the column 
% toColumn in an excel file with name filename.
%NEEDS THE FORMAT OF THE CELLS TO BE CONSTANT IN THIS ORDER:
%Time (s), Compressor speed (rpm), DV-pos (%), p1 (comp inlet press, mbar), 
%T1 (comp inlet temp, C), p2 (mbar), T2 (mbar), Head (poly, kJ/gh), 
%Efficiency (poly, %), mass flow (kg/s), Volumetric flow (m3/s), 
% power (compressor, kW)
TransientData = xlsread(filename, sheetName);
%The ASCII letter 'A' has a value of 65. Thus, if the first column is to be
%reached (or column 'A'), the data must be gathered from column 'A' - 64 as
%that makes 1.
temp = 0; %a temporary variable
if length(fromColumn) > 1 %after 'Z', excel begins on 'AA', then after 26 more columns comes 'BA'
    for i = 1:length(fromColumn) - 1
        temp = temp + (fromColumn(i) - 64)*26; %as 'A' is 65, subtracting 64 would mean 
                %this loop repeats once if A is the first letter, twice if 
                %B is the first letter, etc.
    end %for i = length(fromColumn)
    temp = temp + fromColumn(end);
    fromColumn = temp;
end %if length(fromColumn) > 1

Time = TransientData(:, fromColumn - 64); % s
CompSpeedRPM = TransientData(:, fromColumn - 63); % RPM
DVpos = TransientData(:, fromColumn - 62); % -
p1 = TransientData(:, fromColumn - 61); % mbar
T1 = TransientData(:, fromColumn - 60); % C
p2 = TransientData(:, fromColumn - 59); % mbar
T2 = TransientData(:, fromColumn - 58); % C
polyHead = TransientData(:, fromColumn - 57); % kJ/kg
etaP = TransientData(:, fromColumn - 56); % -
m = TransientData(:, fromColumn - 55); % kg/s
Q = TransientData(:, fromColumn - 54)/(60 * 60); % m3/s, but given in m3/h
pwr = TransientData(:, fromColumn - 53); % kW

%No current use for 'toColumn'

end %function DVdataExtracter