%% Prescript and about
close all
clear
clc

%Created by Sander K. Arstad,  Autumn, 2021
%This script attempts to organise all 'Transient Response'-readings in one
%document for ease of access. Mufch of this script is similar (read:
%exactly the same) as the soft sensors script.

%% Importing data
run('NTNUExperimentalReader.m');
%NTNUExperimentalReader('CASE1Slow_5000_V70.tdms');

% All control of whats being studied is happening in the 
%NTNUExperimentalReader script.

%% Constants
D_iO1 = 0.25; %m, inner tube diameter before/after orifice1
D_iO2 = 0.23; %m, inner tube diameter before/after orifice2
beta1 = 0.640072; %d/D, -, beta, first orifice. Either 0.640072, or 0.40
beta2 = 130.176 / (D_iO2*1000); %d/D, -, beta, second orifice. D might be innacurrate

R0_air = 287.058; % J/(kg K), specific gas constant for air
R0_water = 461.5; % J/(kg K), specific gas constant for water

nu_air = 1.562 * 10^(-5); % m^2/s, kinematic viscosity for air (at 25 deg C) 
rho_air = 1.184; % kg/m^3, density before orifice (for air at 25 deg C)
%both^ according to https://www.engineersedge.com/physics/viscosity_of_air_dynamic_and_kinematic_14483.htm
%% Variables
R0 = R0_air ; %No water being used
nu = nu_air ; %No water being used
sensorsO1.time = pxiTime;
sensorsO2.time = pxiTime;
sensorsO1.rho = rho_air; %No water being used
sensorsO2.rho = sensorsO1.rho; %Density for orifice 2
sensorsO1.logFreq = loggingFreq; %hZ
sensorsO2.logFreq = loggingFreq; %hZ

%%%Average variables (for easy access during verification)
AveInletTemp = mean((TempInlet16 + TempInlet17 + TempInlet18 + TempInlet19)/4);
AveOutletTemp = mean((TempOutlet20 + TempOutlet21 + TempOutlet22 + TempOutlet23)/4);
AveInletPress = mean((PressureInletA + PressureInletB + ...
    PressureInletC + PressureInletD)/4);
AveOutletPress = mean((PressureOutletA + PressureOutletB + ...
    PressureOutletC)/3);% + PressureOutletD)/4);

%%%Orifice variables
    kappa = 1.401; % -, specific heat ratio, approximate at 1.4, should calculate
    %First Orifice
    TempOrifice1 = TempOrifice1 + 273.15 ; %K, converting from C to K
    p1O1 = (Orifice1InletPressure)*10^2; %Pa, pressure (right) before orifice1
    %Second Orifice
    TempOrifice2 = TempInletA + 273.15 ; %K, converting from C to K
    p1O2 = (Orifice2InletPressure)*10^2; %Pa, pressure (right) before orifice2
    
%%%Compressor variables
    p1 = (PressureInletA + PressureInletB + PressureInletC + ...
        PressureInletD)/4 * 10^2; %Pa, average inlet pressure of compressor
    p2 = (PressureOutletA + PressureOutletB + ...
        PressureOutletC )/3 * 10^2; % Pa, average outlet pressure of compressor, 
        %PressureOutletD disregarded as it was too far away from the cluster
        %+ PressureOutletD)/4; %Pa, average outlet pressure of compressor
    if cTemps == 0
        T1 = (TempInlet16 + TempInlet17 + TempInlet18 + TempInlet19)/4 ...
            + 273.15; %K, temperature before compressor
        T2 = (TempOutlet20 + TempOutlet21 + TempOutlet22 + TempOutlet23)/4 ...
            + 273.15; %K, temperature after compressor
    elseif cTemps == 1
        T1 = (cleanTempInlet16 + cleanTempInlet17 + cleanTempInlet18 ...
            + cleanTempInlet19)/4 + 273.15; %K, temperature before compressor
        T2 = (cleanTempOutlet20 + cleanTempOutlet21 + cleanTempOutlet22 ...
            + cleanTempOutlet23)/4 + 273.15; %K, temperature after compressor
    else
        error('Choose a valid value for cTemps (0 or 1)');
    end %if cTemps == 0
    TempIter = length(TempInlet16); %Number of iterations in the original Temp-rawdata
    
    Z1 = 1; %ASSUMING IDEAL GAS (ok assumption for air @ 300K, 1 bar)
    Z2 = 1; %ASSUMING IDEAL GAS (less-ok, but rho2 is not used in a critical aspect)

%Comment below explains the thought-process for a specific incident, but 
%the solution is generalised to work for most incidents of similar nature
    %As temps are measured every 12s, and pressure every 0.5s, a temp value
    %must be repeated 24 times before a new temp-value  occurs
    ok = 1; %is the above statement ok to use? If yes, write 1 for ok, and
            % it will be used, else, write 0
    RepetitionNumber = floor(length(p1)/length(T1)) + 1; %The number of times 1 T-element needs to be repeated

switch ok %switch-statement is for different methods of selecting T-values for the compressor
    case 1 %true
        T1_Temporary = zeros(1, length(p1)); %preallocates
        T2_Temporary = zeros(1, length(p2)); %preallocates
        T1(end+1) = T1(end); %makes sure the index in the for-loop does not exceed array elements
        T2(end+1) = T2(end); %makes sure the index in the for-loop does not exceed array elements
        for i = 1:RepetitionNumber:length(p1)-1
             T1_Temporary(i:(i + RepetitionNumber)) = T1(1 + floor(i/RepetitionNumber));
             T2_Temporary(i:(i + RepetitionNumber)) = T2(1 + floor(i/RepetitionNumber));
         end % for
         T1 = T1_Temporary;
         T2 = T2_Temporary;
         if (length(T1) == length(T2)) && (length(p1) == length(p2))
             if (length(T1) > length(p1))
                 %Problem; T1 > p1. Solution; remove excess T1 elements
                 for j = 1:(length(T1) - length(p1))
                     T1(length(T1)) = [];
                     T2(length(T2)) = [];
                 end%for
             elseif (length(T1) < length(p1))
                 %Problem; T1 < p1. Solution; remove excess p1 elements
                 for j = 1:(length(p1) - length(T1))
                     p1(length(p1)) = [];
                     p2(length(p2)) = [];
                 end
             end %if T1 > p1
         else %Displaying error messages to indicate more specific coding is needed
             disp("Is length(T1) == length(T2)?");
             disp(length(T1) == length(T2));
             disp("Is length(p1) == length(p2)?");
             disp(length(p1) == length(p2));
             error('Something is wrong, see above. 0 = wrong.');
         end%if T1 == T2
     case 0 %false
        %compressor inlet temperature is averaged for entire test
        %across all measuring tools selected. NOT APPLCABLE DURING
        %TRANSIENT CONDITIONS
        for n = 1:length(p1)
            T1(n) = AveInletTemp + 273; % K
            T2(n) = AveOutletTemp + 273; % K
        end %for n = 1:length(p1)
    otherwise
        msg = ["Must select between either ok, or not ok", ...
            + " (conscientiously). READ the comment and thereafter", ...
            + " CHOOSE conscientously."];
        error(msg);
end %switch
    
    rho1 = p1./(R0 .* T1 .* Z1); % kg/m^3, density before compressor
    rho2 = p2./(R0 .* T2 .* Z2); % kg/m^3, density after compressor
    
    
    

%% Formulae

%%% Massflow by Orifice and Venturi
%Orifice calculation according to ISO5167-2:2003
tolO = 10^-3; %A tolerance for determining when v has converged

    %First Orifice calculation
sensorsO1.name = ["DP25", "DP250", "DP Smar"]';
sensorsO1.maxDP = [25, 250, 100];
sensorsO1.DP = [ Orifice1DP25; Orifice1DP250; Orifice1Smar];
for iter = 1:length(sensorsO1.name)
end% for iter = 1:length
sensorsO1.Q = ones(length(sensorsO1.name), length(p1O1)) * 1; %m^3/s, volumetric flow, a first-guess
sensorsO1.Re_D = sensorsO1.Q ; % preallocating
Re = 4 / (pi * D_iO1 * nu_air);
[row, ~] = size(sensorsO1.Q);
for i = 1:row
    sensorsO1.Re_D(i,:) = sensorsO1.Q(i,:) .* Re;
end % for i = row
%v_prevS1 = 0;  v_prevS2 = 0; %m/s, previous velocity (for velocity iteration)
d_iO1 = beta1 * D_iO1 ; % m, smallest inner diameter over orifice
iterOrifice1 = zeros(1, row); % -, counts the number of iterations needed in the orifice while-loop

% --- %
%Must find something that works for a general purpose of this. Was done,
%because TempOrifice1 was never found to be changing significantly over the
%couse of the experiments.
TempOrifice1(1, 2) = mean(TempOrifice1);
% --- %
%recording the negative factors
[negativeFactorsO1, sensorsO1.DP] = ReversedFlowDetector(sensorsO1.DP); 
%sensor range
sensorsO1.range = zeros(length(sensorsO1.maxDP), 2); %preallocating
for iter = 1:length(sensorsO1.name)
    sensorsO1.range(iter, :) = [-sensorsO1.maxDP(iter), sensorsO1.maxDP(iter)]; %range of the sensors    
    sensorsO1.p2(iter, :) = (Orifice1InletPressure - sensorsO1.DP(iter, :))*10^2; % Pa, pressure after orifice
end

%preallocating
sensorsO1.A = zeros(length(sensorsO1.name), length(p1O1));   sensorsO1.C = sensorsO1.A; ...
sensorsO1.kappa = sensorsO1.A;      sensorsO1.epsilon = sensorsO1.A; ...
sensorsO1.m = sensorsO1.A;  ...
sensorsO1.rho = p1O1 ./ (R0 .* TempOrifice1(1) .* Z1);%:length(TempOrifice)-1));



%Mass flow calculation of first orifice
for j = 1:length(sensorsO1.name)
    Re_prev = zeros(1, length(p1O1));
while (abs(mean(sensorsO1.Re_D(j, :) - Re_prev)) > tolO)
    iterOrifice1(j) = iterOrifice1(j) + 1;
    %Re_DS1 = D_iO1 * vO1S1 / nu; % -, Reynolds number for orifice
    %Re_DS2 = D_iO1 * vO1S2 / nu; % --||--, sensor 2
    
    sensorsO1.A(j, :) = (19000 .* beta1 ./ sensorsO1.Re_D(j, :)).^(0.8); %-
    sensorsO1.C(j, :) = 0.5961 + 0.0261 * beta1^2 - 0.216 * beta1^8 + ...
        0.000521 * (beta1 * 10^6 ./ sensorsO1.Re_D(j, :)).^(0.7) + ...
        (0.0188 + 0.0063 .* sensorsO1.A(j, :)) .* beta1^(3.5) .* ...
        (10^6./sensorsO1.Re_D(j, :)).^(0.3);% -, simplified
    sensorsO1.kappa(j, :) = ( (1 - beta1^4 .*(1 - sensorsO1.C(j, :).^2)).^(0.5) ...
        ./(sensorsO1.C(j, :) .* beta1^2) - 1 ).^2;
    sensorsO1.epsilon(j, :) = 1- (0.351 + 0.256.*beta1.^4 + ...
        0.93.*beta1.^8) .* ...
        (1 - (sensorsO1.p2(j, :)./p1O1).^(1./sensorsO1.kappa(j, :))); % -
    sensorsO1.m(j, :) = negativeFactorsO1(j,:) .* (sensorsO1.C(j, :) ./ sqrt(1 - beta1^4)) .* ...
        sensorsO1.epsilon(j, :).* (pi/4) .* d_iO1^2 .* ...
        sqrt(2 .* sensorsO1.DP(j, :) * 10^2 .* sensorsO1.rho); % kg/s, mass flow through orifice

    sensorsO1.Q(j, :) = (sensorsO1.m(j, :) ./ sensorsO1.rho); %m3/s, volumetric flow
    
    %updating reynolds number
    Re_prev = sensorsO1.Re_D(j,:);
    sensorsO1.Re_D(j, :) = abs(4 * sensorsO1.Q(j, :) / (pi * D_iO1 * nu_air));
            %absolute values here, because the code can't handle negative
            %values for Re_D, and it is only needed for m and Q
            %calculations anyways...
    if (iterOrifice1(j) > 1000)
        msg = ["Orifice iteration exceeded " + iterOrifice1(j) + " number of iterations"];
        error(msg);
    end %if iterOrifice > 1000;
end %while

end %for j = 1:length(sensorsO1.name)

%Adding the negative readings back to pressure readings (no longer abs. DP
%pressure)
sensorsO1.DP = sensorsO1.DP .* negativeFactorsO1;
%------------------------------------------------------------------------%

    %Second Orifice calculation
sensorsO2.name = ["DP25", "DP50", "DP250", "DP smar"]';%, "DP10", "DPsmar"]; %sensorsO2.name = ["DP25", "DP250"]'; 
sensorsO2.maxDP = [25, 50, 250, 250]; %sensorsO2.maxDP = [25, 250]; 
sensorsO2.DP = [ Orifice2DP25; Orifice2DP50; Orifice2DP250; Orifice2Smar]; %sensorsO2.DP = [ Orifice2DP50; Orifice2DP250];

sensorsO2.Q = ones(length(sensorsO2.name), length(p1O2)) * 1; %m^3/s, volumetric flow, a first-guess
sensorsO2.Re_D = sensorsO2.Q; %Preallocating
Re = 4 / (pi * D_iO2 * nu_air);
[row, ~] = size(sensorsO2.Q);
for i = 1:row
    sensorsO2.Re_D(i,:) = sensorsO2.Q(i,:) .* Re;
end %for i = row
d_iO2 = beta2 * D_iO2 ; % m, smallest inner diameter over orifice
iterOrifice2 = zeros(1, row); % -, counts the number of iterations needed in the orifice while-loop

% --- %
%MÅ finne ut noe som virker generelt for denne
%TempOrifice2 = TempInletA + 273; %K
% --- %
[negativeFactorsO2, sensorsO2.DP] = ReversedFlowDetector(sensorsO2.DP);

sensorsO2.range = zeros(length(sensorsO2.maxDP), 2); %preallocating
for iter = 1:length(sensorsO2.maxDP)
    sensorsO2.range(iter, :) = [-sensorsO2.maxDP(iter), ...
        sensorsO2.maxDP(iter)]; %range of the sensors
    sensorsO2.p2(iter, :) = (Orifice2InletPressure - ...
        sensorsO2.DP(iter,:))*10^2; % Pa, pressure after orifice
end
%preallocating
sensorsO2.A = zeros(length(sensorsO2.name), length(p1O2));   
sensorsO2.C = sensorsO2.A;          sensorsO2.kappa = sensorsO2.A;      
sensorsO2.epsilon = sensorsO2.A;    sensorsO2.m = sensorsO2.A;  
sensorsO2.rho = p1O2 ./ (R0 .* TempOrifice2(1) .* Z2);%:length(TempOrifice)-1));

%Mass flow calculation of second orifice
for j = 1:length(sensorsO2.name)
    Re_prev = zeros(1, length(p1O2));
while (abs(mean(sensorsO2.Re_D(j, :) - Re_prev)) > tolO)
    iterOrifice2(j) = iterOrifice2(j) + 1;
    %Re_DS1 = D_iO1 * vO1S1 / nu; % -, Reynolds number for orifice
    %Re_DS2 = D_iO1 * vO1S2 / nu; % --||--, sensor 2
    sensorsO2.A(j, :) = (19000 .* beta2 ./ sensorsO2.Re_D(j, :)).^(0.8); %-
    sensorsO2.C(j, :) = 0.5961 + 0.0261 * beta2^2 - 0.216 * beta2^8 + ...
        0.000521 * (beta2 * 10^6 ./ sensorsO2.Re_D(j, :)).^(0.7) + ...
        (0.0188 + 0.0063 .* sensorsO2.A(j, :)) .* beta2^(3.5) .* ...
        (10^6./sensorsO2.Re_D(j, :)).^(0.3);% -, simplified
    sensorsO2.kappa(j, :) = ( (1 - beta2^4 .*(1 - sensorsO2.C(j, :).^2)).^0.5 ...
        ./(sensorsO2.C(j, :) .* beta2^2) - 1 ).^2;
    sensorsO2.epsilon(j, :) = 1- (0.351 + 0.256.*beta2.^4 + ...
        0.93.*beta2.^8) .* ...
        (1 - (sensorsO2.p2(j, :)./p1O2).^(1./sensorsO2.kappa(j, :))); % -
    sensorsO2.m(j, :) = negativeFactorsO2(j,:) .* (sensorsO2.C(j, :) ./ sqrt(1 - beta2^4)) .* ...
        sensorsO2.epsilon(j, :).* (pi/4) .* d_iO2^2 .* ...
        sqrt(2 .* sensorsO2.DP(j, :)*10^2 .* sensorsO2.rho); % kg/s, mass flow through orifice

    sensorsO2.Q(j, :) = (sensorsO2.m(j, :) ./ sensorsO2.rho); %m3/s, volumetric flow
    
    %updating reynolds number
    Re_prev = sensorsO2.Re_D(j,:);
    sensorsO2.Re_D(j, :) = abs(4 * sensorsO2.Q(j, :) / (pi * D_iO2 * nu_air)); 
            %absolute values here, because the code can't handle negative
            %values for Re_D, and it is only needed for m and Q
            %calculations anyways...

    if (iterOrifice2(j) > 1000)
        msg = ["Orifice iteration exceeded " + iterOrifice2(j) + " number of iterations"];
        error(msg);
    end %if iterOrifice > 1000;
end %while

end %for j = 1:length(sensorsO2.name)

%Adding the negative readings back to pressure readings (no longer abs. DP
%pressure)
sensorsO2.DP = sensorsO2.DP .* negativeFactorsO2;

%End orifice calculation
%end mass flow calculation
%-----------------------------------------------------------------------%

%%% Compressor calculation
    %%%% isentropic calculation
    %As of 06.12, something is very wrong. The iterative process is not at
    %all iterative. The problem is that the current way kappaT is being 
    %calculated just means kappaT = kappa, instead of actually calculating
    %anything and iterating. Another, different equation is needed for this
    %to make sense. Something for later work.
%tolT = tolO; %tolerance for T2s while-loop
%T2s = T1 .* (p2./p1).^((kappa-1)./kappa); % K, initial guess, corrected in while-loop
%T2s_prev = 0; %initiates T2s while-loop
%iterT2s = 0; %iteration counter in T2s loop

%while abs(T2s - T2s_prev) > tolT*10^-10
%    iterT2s = iterT2s + 1;
%    Ln = log(T2s./T1)./log(p2./p1);
%    kappaT = 1./(1 - Ln);
    
    
%    T2s_prev = T2s;
%    T2s = T1 .* (p2./p1).^((kappaT-1)./kappaT);
%    if(iterT2s > 1000)
%        msg = ["T2s iteration exceeded " + iterT2s " number of iterations"];
%        error(msg);
%    end %if iterT2s > 1000
%end %while T2s not converged

    
%rho2s = p2./(R0 .* T2s .* Z2); %kg/m^3, density after isentropic compression
        %NOTE: Z2 not defined (currently (04.12.21)), Z2 = 1. Maybe look
        %into defining it properly?
%kappaV = -(log(p2./p1))./(log(rho2s./rho1)); % -, specific volume exponent
%Hs = kappaV./(kappaV-1) .* Z1 .* R0 .* T1 .* ...
%    ( (p2./p1).^((kappaV-1)./kappaV) -1); %J/kg, = h2s - h1, isentropic head 
   %end isentropic calculation
   
   %%%% polytropic calculation
%n_v = (log(p2./p1) ./ log(rho2./rho1)); % -, polytropic volume exponent
 %f = (Hs) ./ ((kappaV/(kappaV-1)) * (p2/rho2s - p1/rho1)); % -, compressor 
                    %correction factor according to Schultz. Seems off

%Polytropic head:
%Hp = ((n_v./(n_v-1)) .* (Z1 .* R0 .* T1) .* ...
%    ( (p2./p1).^((n_v-1)./n_v) -1 )); %.* f; % J/kg, polytropic head


%Actual head: H = h2 - h1
%CpMethod = 'o'; %o for Ohio, or W for Wark.
%if CpMethod == 'W'
%Calculating Cp variance according to K. Wark, Thermodynamics, 4th edition,
%McGraw-Hill, New York, 1983, as based on NASA SP-273, U.S. Government
%Printing Office, Washington DC, 1971. See function CpVarianceFunc for more
%details.
%    Cp1 = CpVarianceFunc(T1, (R0*10^-3));
%    Cp1_2 = CpVarianceFunc((T1+T2)./2, R0 * 10^(-3));
%    Cp2 = CpVarianceFunc(T2, (R0*10^-3));
%    CpAve = (Cp1 + 2*Cp1_2 + Cp2)/4;
%elseif CpMethod == 'o'
%Using
%https://www.ohio.edu/mechanical/thermo/property_tables/air/air_Cp_Cv.html
%as the temperature variance is within 50 deg C.
%    CpAve = (1.005 + 1.008)/2; %Cp of air at 300K, and 350K, respectively.
%    CvAve = (0.718 + 0.721)/2; %CV of air at 300K, and 350K, respectively.
%    kappa = CpAve/CvAve;
%end % method Cp


%H = CpAve .* (T2 - T1).*10.^3; %J/kg, Strictly, NOT REALLY VALID HERE, only
                % possible for a constant pressure process, as Cp changes 
                % with changes in pressure 

%Polytropic efficiency, calculated in 3 different ways:
%etaP = Hp./H; %"Standard way" (definition)
%etaP2 = (kappa - 1)/kappa .* log(p2./p1)./log(T2./T1); %Example 1.6 from Turbomachinery class
%etaP3 = (kappa - 1)./kappa .* n_v./(n_v - 1); % A third way of calculating etaP
%etaPTorque = 0;%calculate polytropic efficiency based on torque delivered by 
    %the compressor (compr. RPM x )

%dyTol = 0.05 * 10^-2;
%[etaPaveTime, etaPave] = jaggedDataSmoother(pxiTime, etaP, dyTol);
%[etaP3aveTime, etaP3ave] = jaggedDataSmoother(pxiTime, etaP3, dyTol);
%"jaggedDataSmoother" is a function which takes the average of jagged data
%defined by: 
%[xData, yDataAverage] = jaggedDataSmoother(xData, yDataJagged, jagg-tolerance)
% (see "functions"-part of script for function definition). The jagged data
% is a result of the temperature extension done near the start of this
% script, where multiple temperature data-points were duplicated because of
% the slower measuring frequency for temperature. Smoothing this data by
% taking the average of the interval creates a curve which is easier to
% see, and a lot more cleaner than the jagged datapoints produced by
% duplicating the same temperature reading for different pressure readings.

    %end polytrsopic calculation

%End compressor calculation

%% Recording sensor-specific output
ut = [mean(shaftSpeedRPM), mean(DVpos), (mean(mean(sensorsO1.Q)) + ...
    mean(sensorsO2.Q(2,:)))/2, mean(p2./p1), pxiTime(end)];

Pr = p2./p1; %pressure ratio
O1DP1 = data.PXI_0.DP1_Orifice1.data; %This is +- 250 mbar
O1DP2 = data.PXI_0.DP2_Orifice1.data; %This is  +- 25 mbar
O1Smar = data.PXI_0.DP_Smar_Orifice1.data; %This is DP smar on OR1
O2DP1 = data.PXI_0.DP1_Orifice2.data; %This is  +- 250 mbar
O2DP2 = data.PXI_0.DP2_Orifice2.data; %This is +- 50 mbar
O2DP3 = data.PXI_0.DP3_Orifice2.data; %This is +- 25 mbar
O2Smar = data.PXI_0.DP_Smar_Orifice2.data; %This is DP smar on OR2

%General Q and m calculations
sensors = 7;
% 1 means ALL DP25 sensors used. 2 means Orifice 1 DP25 sensor used, Or2
% DP25 excluded. 3 means no DP25 sensors are used.
switch sensors
    case 1 %Or1: All sensors. Or2: All sensors
        Q_Or1 = mean(mean(sensorsO1.Q));
        Q_Or2 = mean(mean(sensorsO2.Q));
        Q = (Q_Or1 + Q_Or2)/2;

        Q_Or1_vec = mean(sensorsO1.Q);
        Q_Or2_vec = mean(sensorsO2.Q);
        Q_vec = (Q_Or1_vec + Q_Or2_vec) /2;
        
        m_Or1 = mean(mean(sensorsO1.m));
        m_Or2 = mean(mean(sensorsO2.m));
        m = (m_Or1 + m_Or2)/2;
    case 2 %Or1: All sensors. Or2: All but DP25.
        Q_Or1 = mean(mean(sensorsO1.Q));
        Q_Or1_vec = mean(sensorsO1.Q);
        m_Or1 = mean(mean(sensorsO1.m));

        Q_Or2 = 0; Q_Or2_vec = 0; m_Or2 = 0; j = 0;
        for i = 1:length(sensorsO2.name)
            if (sensorsO2.name(i) ~= "DP25")
                Q_Or2 = Q_Or2 + mean(sensorsO2.Q(i,:));
                Q_Or2_vec = Q_Or2_vec + sensorsO2.Q(i,:);
                m_Or2 = m_Or2 + mean(sensorsO2.m(i,:));
                j = j + 1;
            end % if sensorsO2.name(i) ~= "DP25"
        end %for i = length(sensorsO1.name)
        Q_Or2 = Q_Or2 / j;
        Q_Or2_vec = Q_Or2_vec / j;
        m_Or2 = m_Or2 / j;

        Q = (Q_Or1 + Q_Or2)/2;
        Q_vec = (Q_Or1_vec + Q_Or2_vec)/2;
        m = (m_Or1 + m_Or2)/2;

    case 3 %Or1: All but DP25. Or2: All but DP25.
        Q_Or1 = 0; Q_Or2 = 0;   Q_Or1_vec = 0; Q_Or2_vec = 0; 
        m_Or1 = 0; m_Or2 = 0;   j = 0;
        for i = 1:length(sensorsO1.name)
            if (sensorsO1.name(i) ~= "DP25")
                Q_Or1 = Q_Or1 + mean(sensorsO1.Q(i,:));
                Q_Or1_vec = Q_Or1_vec + sensorsO1.Q(i,:);
                m_Or1 = m_Or1 + mean(sensorsO1.m(i,:));
                j = j + 1;
            end% if sensorsO1.name(i) ~= "DP25"
        end %for i = length(sensorsO1.name)
        Q_Or1 = Q_Or1 / j;
        Q_Or1_vec = Q_Or1_vec / j;
        m_Or1 = m_Or1 / j;
        j = 0;
        for i = 1:length(sensorsO2.name)
            if (sensorsO2.name(i) ~= "DP25") %must check if this ever fires
                Q_Or2 = Q_Or2 + mean(sensorsO2.Q(i,:));
                Q_Or2_vec = Q_Or2_vec + sensorsO2.Q(i,:);
                m_Or2 = m_Or2 + mean(sensorsO2.m(i,:));
                j = j + 1;
            end % if sensorsO2.name(i) ~= "DP25"
        end %for i = length(sensorsO1.name)
        Q_Or2 = Q_Or2 / j;
        Q_Or2_vec = Q_Or2_vec / j;
        m_Or2 = m_Or2 / j;
        
        Q = (Q_Or1 + Q_Or2)/2;
        Q_vec = (Q_Or1_vec + Q_Or2_vec)/2;
        m = (m_Or1 + m_Or2)/2;

    case 4 %Or 1: All sensors. Or 2: DP250, DPsmar
        Q_Or1 = mean(mean(sensorsO1.Q));
        Q_Or2 = mean(sensorsO2.Q(3, :) + sensorsO2.Q(4,:))/2;
        Q = (Q_Or1 + Q_Or2)/2;

        Q_Or1_vec = mean(sensorsO1.Q);
        Q_Or2_vec = (sensorsO2.Q(3,:) + sensorsO2.Q(4,:))/2;
        Q_vec = (Q_Or1_vec + Q_Or2_vec) /2;
        
        m_Or1 = mean(mean(sensorsO1.m));
        m_Or2 = mean(sensorsO2.m(3, :) + sensorsO2.m(4,:))/2;
        m = (m_Or1 + m_Or2)/2;


    case 5 %Or 1: DP250, DPsmar. Or2: DP250, DPsmar
        Q_Or1_vec = (sensorsO1.Q(2, :) + sensorsO1.Q(3, :))/2;
        Q_Or2_vec = (sensorsO2.Q(3, :) + sensorsO2.Q(4, :))/2;
        Q_vec = (Q_Or1_vec + Q_Or2_vec)/2;

        Q_Or1 = mean(Q_Or1_vec);
        Q_Or2 = mean(Q_Or2_vec);
        Q = (Q_Or1 + Q_Or2)/2;
        
        m_Or1 = mean(sensorsO1.m(2, :) + sensorsO1.m(3,:))/2;
        m_Or2 = mean(sensorsO2.m(3, :) + sensorsO2.m(4,:))/2;
        m = (m_Or1 + m_Or2)/2;

    case 6 %Or 1: DP250, DPsmar. Or 2: Only DP250
        Q_Or1_vec = (sensorsO1.Q(2, :) + sensorsO1.Q(3, :))/2;
        Q_Or2_vec = sensorsO2.Q(3,:);
        Q_vec = (Q_Or1_vec + Q_Or2_vec)/2;

        Q_Or1 = mean(Q_Or1_vec);
        Q_Or2 = mean(Q_Or2_vec);
        Q = (Q_Or1 + Q_Or2)/2;
        
        m_Or1 = mean(sensorsO1.m(2, :) + sensorsO1.m(3,:))/2;
        m_Or2 = mean(sensorsO2.m(3, :));
        m = (m_Or1 + m_Or2)/2;
    case 7 %Or1: all sensors, Or2: ONLY DP50 USED
        Q_Or1 = mean(mean(sensorsO1.Q));
        Q_Or2 = mean(sensorsO2.Q(2, :));
        Q = (Q_Or1 + Q_Or2)/2;

        Q_Or1_vec = mean(sensorsO1.Q);
        Q_Or2_vec = sensorsO2.Q(2,:);
        Q_vec = (Q_Or1_vec + Q_Or2_vec) /2;
        
        m_Or1 = mean(mean(sensorsO1.m));
        m_Or2 = mean(sensorsO2.m(2, :));
        m = (m_Or1 + m_Or2)/2;
end %switch sensors

output = [mean(shaftSpeedRPM), mean(DVpos), Q, Q_Or1, Q_Or2, ...
   mean(Pr), m, m_Or1, m_Or2, ...
   mean(XT3_1P), mean(XT3_1T), mean(XT3_1R), mean(O1DP1), ...
   mean(O1DP2), mean(O1Smar), mean(O2DP1), mean(O2DP2), mean(O2DP3)...
   mean(O2Smar), pxiTime(end)];
% fprintf("Property: \t\t\t (Mean) value:\nRPM \t\t\t\t %f\nDV pos" +...
%     " \t\t\t\t %f\nQ \t\t\t\t\t %f\nPr \t\t\t\t\t %f\n" +...
%     "m \t\t\t\t\t %f\n----\n" +...
%     "P_bar \t\t\t\t %f\nT_bar \t\t\t\t %f\nHumid. \t\t\t\t %f\n ----"+...
%     "\nOr1DP25 \t\t\t %f\nOr1DP250 \t\t\t %f\nOr1DPSmar \t\t\t %f\n"+...
%     "Or2DP25 \t\t\t %f\nOr2DP50 \t\t\t %f\nOr2DP250 \t\t\t %f\n"+...
%     "Or2DPSmar \t\t\t %f\n", output); 
% 
% Q_and_m_only = [Q, m];


%% Tables
%Commented out, until Hp is defined properly
%fprintf('\n\n\t\t\t\tAverage Matlab results:\n');
%Property = ['Rel. humidity'; 'Barom. Temp. '; 'Barom. Press.'; ...
%    'Inlet Temp.  '; 'Inlet Press. ';'Outlet Temp. '; 'Outlet Press.';...
%    'Mass flow    '; '-------------'; 'Poly. Head   '; 'Actual Head  ';...
%    'Poly. Eff.   '];
%Results = [mean(XT3_1R); mean(XT3_1T); mean(XT3_1P)*10^3; ...
%    mean(T1)-273; mean(p1)*10^-2;...
%    mean(T2)-273; mean(p2)*10^-2; mean(sensorsO1.m(1,:)) ; 0; mean(Hp)*10^-03;...
%    mean(H)*10^-03; mean(etaP)*10^+2];
%Unit = ['%    '; 'deg C'; 'mbar '; 'deg C'; 'mbar '; 'deg C';...
%    'mbar '; 'kg/s '; '-----'; 'kJ/kg'; 'kJ/kg'; '%    '];

%PropertyTab = table(Property, Results, Unit);
%disp(PropertyTab);

%Mass flow difference
    %max difference
[maxMassFlowDiff1, maxMassFlowDiff1Pos] = ...
    max(abs(sensorsO1.m(1, :) - sensorsO2.m(1, :)));
    
[maxMassFlowDiff2, maxMassFlowDiff2Pos] = ...
    max(abs(sensorsO1.m(2, :) - sensorsO2.m(3, :)));

relMaxDiff1 = maxMassFlowDiff1 / max(sensorsO1.m(1, maxMassFlowDiff1Pos), ...
    sensorsO2.m(1, maxMassFlowDiff1Pos)) * 100; % %, greatest relative 
                            % difference of the mass flow calculated values
relMaxDiff2 = maxMassFlowDiff2 / max(sensorsO1.m(2, maxMassFlowDiff2Pos), ...
    sensorsO2.m(3, maxMassFlowDiff2Pos)) * 100; % %, greatest relative 
                            % difference of the mass flow calculated values
                            
    %average difference
meanMassFlowDiff1 = mean(abs(sensorsO1.m(1, :) - sensorsO2.m(1, :)));
meanMassFlowDiff2 = mean(abs(sensorsO1.m(2, :) - sensorsO2.m(3, :)));

relMeanDiff1 = meanMassFlowDiff1 / ... 
    max(max(sensorsO1.m(1, :), sensorsO2.m(1, :))) * 100; %the relative mean flow
                        % difference when measured against the largest flow
relMeanDiff2 = meanMassFlowDiff2 / ...
    max(max(sensorsO1.m(2, :), sensorsO2.m(3, :))) * 100;

fprintf("\n\nMass flow difference:\n");
ranges = [25; 250];
maxMassFlowDiff = [maxMassFlowDiff1; maxMassFlowDiff2];
relativeMaxFlowDiff = [relMaxDiff1; relMaxDiff2];
meanMassFlowDiff = [meanMassFlowDiff1; meanMassFlowDiff2];
relativeMeanFlowDiff = [relMeanDiff1; relMeanDiff2];

massFlowTable = table(ranges, maxMassFlowDiff, relativeMaxFlowDiff, ...
    meanMassFlowDiff, relativeMeanFlowDiff);
disp(massFlowTable)


%% Plots
%Plotting Isentropic Head and Temperature covariance
%IsentropicHeadTemperatureCovariance(T1, Hs)

if (RPMTransient || DVTransient)
    %pxiTime is relevant time-axis for Hp and etaP, as H is dependant on p,
    %which is read from pxi
    
    % COMPRESSOR HEADS COMMENTED OUT UNTIL Hp IS CALCULATED PROPERLY
    
    %leg = ""; %preallocating
    %figure %Plotting Compressor heads
    %if simMatch
    %    plot(simTime, simHead)
    %    hold on
    %    leg = "Simulation";
    %end %if simMatch
    %plotTime(Hp * 10^(-3), 'Head, $kJ/kg$', ...
    %    'Polytropic Compressor Head', pxiTime)
    %if leg == ""
    %    leg = "Calculated";
    %else
    %    leg(end + 1) = "Calculated";
    %end %if leg == ""
    %legend(leg', 'location', 'best')
    
    %leg = ""; %preallocating    
    %figure %Plotting compressor efficiencies
    %if simMatch
    %    plot(simTime, simEta)
    %    hold on
    %    leg = "Simulation";
    %end% if simMatch
%     plotTime(etaP * 100, '','', pxiTime)
%     hold on
%     if leg == ""
%         leg = "Calculated, Hp/H";
%     else
%         leg(end + 1) = "Calculated, Hp/H";
%     end %if leg == ""
%     plotTime(etaP2 * 100, '','', pxiTime)
%     hold on
%     leg(end + 1) = "Calculated, $\eta_P2$";
%     plotTime(etaP3 * 100, '$\eta_p$, $\%$', 'Polytropic Efficiency', ...
%         pxiTime)
%     leg(end + 1) = "Calculated, $\eta_P3$";
%     legend(leg', 'location', 'best')
%     
%     
%     leg = "";
%     figure %plotting average experimental efficiencies (and sim data, if available)
%     if simMatch
%         plot(simTime, simEta)
%         hold on
%         leg = "Simulation data";
%     end %if simMatch
%     plotTime(etaPave * 100, '', '', etaPaveTime)
%     hold on
%     if leg == ""
%         leg = "Expr. ave. (Hp/H) $\eta P$";
%     else
%         leg(end + 1) = "Expr. ave. (Hp/H) $\eta P$";
%     end %if leg == ""
%     plotTime(etaP3ave * 100, '$\eta_p$, $\%$', 'Polytropic Efficiency', ...
%         etaP3aveTime)
%     leg(end + 1) = "Expr. ave. $\eta P$";
%     legend(leg', 'location', 'best')
    
    
    leg = "";
    %pxiTime is relevant time-axis for p1 and p2, as p is read from pxi
    figure %plotting (average) output temperature.
    if simMatch
        plot(simTime, simT2)
        hold on
        leg = "Simulation";
    end %if simMatch
    plotTime(T2-273, 'Temperature, $^\circ C$', ...
        'Outlet temperature development', pxiTime)
    if leg == ""
        leg = "Experimental";
    else
        leg(end + 1) = "Experimental";
    end %if leg == ""
    legend(leg', 'location', 'best')

    leg = "";
    figure %plotting (average) output pressure.
    if simMatch
        plot(simTime, sim_p2)
        hold on
        leg = "Simulation";
    end % if simMatch
    plotTime(p2 * 10^(-2), 'Pressure, mbar', ...
        'Outlet pressure development', pxiTime)
    if leg == ""
        leg = "Experimental";
    else
        leg(end + 1) = "Experimental";
    end %if leg == ""
    legend(leg', 'location', 'best')
    
    %Orifice 1
    leg = "";
    %pxiTime is relevant time-axis for m and Q, as Q is dependant on m,
    %which is dependant on p, which is read from pxi
    
    figure %plotting mass flow for Orifice 1
    tiledlayout(2, 1)
    %first tile (showing actual mass flow plots for all sensors in Or. 1)
    nexttile
    if simMatch
        plot(simTime, sim_m)
        hold on
        leg = "Simulation mass flow";
    end % if simMatch
	hold on
for iter = 1:length(sensorsO1.name)
    plot(pxiTime, sensorsO1.m(iter, :))
    if leg == ""
        leg = "Or. 1, " + sensorsO1.name(iter);
    else
        leg(end + 1) = "Or. 1, " + sensorsO1.name(iter);
    end %if leg == ""
end %for iter = 1:length(sensorsO1.name)
    xlabel("Time, $s$")
    ylabel("Mass flow, $kg/s$")
    title("Mass flow rate for orifice 1")
    grid on;    grid minor;
    legend(leg', 'location', 'best')
    
    %second tile (showing DP sensors for Or. 1)
    leg = "";
    nexttile
    hold on
for iter = 1:length(sensorsO1.name)
    plot(pxiTime, sensorsO1.DP(iter, :))
    if leg == ""
        leg = sensorsO1.name(iter);
    else
        leg(end + 1) = sensorsO1.name(iter);
    end %if leg == ""
end %for iter = 1:length(sensorsO1.name)
    xlabel("Time, $s$")
    ylabel("DP, $mbar$")
    title("Differential Pressure (DP) sensors for Orifice 1")
    grid on;     grid minor;
    legend(leg', 'location', 'best')
    
    
    %Orifice 2
    leg = "";
    %pxiTime is relevant time-axis for m and Q, as Q is dependant on m,
    %which is dependant on p, which is read from pxi
    
    figure %plotting mass flow for Orifice 2
    tiledlayout(2, 1)
    %first tile (showing actual mass flow plots for all sensors in Or. 2)
    nexttile
    if simMatch
        plot(simTime, sim_m)
        hold on
        leg = "Simulation mass flow";
    end % if simMatch
    hold on
for iter = 1:length(sensorsO2.name)
    plot(pxiTime, sensorsO2.m(iter, :))
    if leg == ""
        leg = "Or. 2, " + sensorsO2.name(iter);
    else
        leg(end + 1) = "Or. 2, " + sensorsO2.name(iter);
    end %if leg == ""
end %for iter = 1:length(sensorsO2.name)
    xlabel("Time, $s$");    ylabel("Mass flow, $kg/s$");
    title("Mass flow rate for orifice 2")
    grid on; grid minor;
    legend(leg', 'location', 'best')
    
    %second tile (showing DP sensors for Or. 2)
    leg = "";
    nexttile
    hold on
for iter = 1:length(sensorsO2.name)
    plot(pxiTime, sensorsO2.DP(iter, :)) 
    if leg == ""
        leg = sensorsO2.name(iter);
    else
        leg(end + 1) = sensorsO2.name(iter);
    end %if leg == ""
end %for iter
    xlabel("Time, $s$");    ylabel("DP, $mbar$")
    title("Differential Pressure (DP) sensors for Orifice 2")
    grid on;     grid minor;
    legend(leg', 'location', 'best')
    
    
    %Combined plot
    leg = "";
    %pxiTime is relevant time-axis for m and Q, as Q is dependant on m,
    %which is dependant on p, which is read from pxi
    figure %plotting mass flow for both Orifice-plates
    tiledlayout(3, 1)
    %first tile (showing actual mass flow plots for all sensors)
    nexttile
    if simMatch
        plot(simTime, sim_m)
        hold on
        leg = "Simulation mass flow";
    end % if simMatch
    hold on
for iter = 1:length(sensorsO1.name)
    plot(pxiTime, sensorsO1.m(iter, :))
    if leg == ""
        leg = "Or. 1, " + sensorsO1.name(iter);
    else
        leg(end + 1) = "Or. 1, " + sensorsO1.name(iter);
    end %if leg == ""
end%for iter = 1:length(sensorsO1.name)
for iter = 1:length(sensorsO2.name)
    plot(pxiTime, sensorsO2.m(iter, :))
    %leg is never = "" in this case, because leg is definetly defined above
    leg(end + 1) = "Or. 2, " + sensorsO2.name(iter); 
end%for iter = 1:length(sensorsO2.name)
    xlabel("Time, $s$");    ylabel("Mass flow, $kg/s$");
    title("Mass flow rate, both orifices")
    grid on;    grid minor;
    legend(leg', 'location', 'best')
    
    %second tile (showing DP sensors for Or. 1)
    leg = "";
    nexttile
    hold on
for iter = 1:length(sensorsO1.name)
    plot(pxiTime, sensorsO1.DP(iter, :))
    if leg == ""
        leg = sensorsO1.name(iter);
    else
        leg(end + 1) = sensorsO1.name(iter);
    end % if leg == ""
end %for iter = 1:length(sensorsO1.DP(1))
    xlabel("Time, $s$");    ylabel("DP, $mbar$");
    title("Differential Pressure (DP) sensors for Orifice 1")
    grid on;    grid minor;
    legend(leg', 'location', 'best')
    
    %third tile (showing DP sensors for Or. 2)
    leg = "";
    nexttile
    hold on
for iter = 1:length(sensorsO2.name)
    plot(pxiTime, sensorsO2.DP(iter, :))
    if leg == ""
        leg = sensorsO2.name(iter);
    else
        leg(end + 1) = sensorsO2.name(iter);
    end % if leg == ""
end %for iter = 1:length(sensorsO2.name)
    xlabel("Time, $s$");    ylabel("DP, $mbar$");
    title("Differential Pressure (DP) sensors for Orifice 2");
    grid on;    grid minor;
    legend(leg', 'location', 'best')
    
    %mass flow rate, both orifices, but "corresponding" sensors plotted
    %alone
    figure
    tiledlayout(2, 1)
    %first tile, showing the first sensors (sensorsO1.m(1, :))
    nexttile
    plot(pxiTime, sensorsO1.m(1, :))
    hold on
    plotTime(sensorsO2.m(1, :), 'Mass flow, $kg/s$', "Sensor " + ...
        sensorsO1.name(1) + " from both orifices", pxiTime)
    leg = ["Or. 1";"Or. 2"];
    legend(leg, 'location', 'best')
    
    %second tile, showing the second sensors (sensorsO1.m(2,:))
    nexttile
    plot(pxiTime, sensorsO1.m(2, :))
    hold on
    plotTime(sensorsO2.m(3, :), 'Mass flow, $kg/s$', "Sensor " + ...
        sensorsO1.name(2) + " from both orifices", pxiTime)
    leg = ["Or. 1";"Or. 2"];
    legend(leg, 'location', 'best')
end % if rpm transient or if dv transient

% Q vs Pr plot
%"local" plot (current Q vs Pr plot)
figure
plot(mean(sensorsO1.Q), Pr)
hold on
plot(mean(sensorsO2.Q), Pr)
plot((mean(sensorsO1.Q) + mean(sensorsO2.Q))/2, Pr)
legName = ["Orifice 1, ", "Orifice 2, ", "Average, "] + testName;
titleName = "Volumetric flow vs. Pressure ratio";
readFilename = "QvsPrGraph.xlsx";
    
%Saving the QvsPr data to the file with name saveFilename

if (saveQPrData)
	saveFilename = 'QvsPrGraph.xlsx';
        if rpm ~= 0 && dv == 0
            saveTitle(1) = "RPM";
            saveTitle(2:length(sensorsO1.Q(1,:))) = rpm;
        elseif rpm == 0 && dv ~= 0
            saveTitle(1) = "DV-pos";
            saveTitle(2:length(sensorsO1.Q(1,:))) = dv;
        elseif rpm == 0 && dv == 0
            saveTitle(1:length(sensorsO1.Q(1,:))) = saveQPrDataName;
        end % if dv ~= 0
switch saveQPrDataSheet %depending on what this value is, the cases below occur
    case "GeneralAverage"
        SaveData(saveTitle, saveData, saveFilename, saveQPrDataSheet)
    case "Orifice"
        SaveData(saveTitle, saveData, saveFilename, "Or1Ave")
        SaveData(saveTitle, saveData, saveFilename, "Or2Ave")
    case "Sensors" %NOT YET IMPLEMENTED
        SaveData(saveTitle, saveData, saveFilename, saveQPrDataSheet)
    otherwise
        error("No valid format for 'saveQPrDataSheet' selected");
end %switch saveQPrDataSheet
end %if saveQPrData

plotQvsPr(readFilename, legName, titleName, 0, 0)

figure %Actual Q vs Pr plot. Showing the result using selected sensors 
            %from above (the switch statement around lines 400 - 500)
plot(Q_Or1_vec, Pr)
hold on
plot(Q_Or2_vec, Pr)
plot(Q_vec, Pr)
legName = ["Orifice 1, ", "Orifice 2, ", "Average, "] + testName;
titleName = "Volumetric flow vs. Pressure ratio";
plotQvsPr(readFilename, legName, titleName, 0, 0)



% Ended up being irrelevant. Might remove later.
% tab.filename = "";
% tab.cNames = ["Sensor names", "Sensor range", "Logging frequency"];
% tab.rNames = ["Or.1, I", "Or.1, II", "Or.1, III", "Or.2, I", "Or.2, II" ...
%     "Or.2, III", "Or.2, IV"];
% tab.data = string(zeros(length(tab.rNames), length(tab.cNames)));
% for iter = 1:length(sensorsO1.name)
%     tab.data(iter, :) = [sensorsO1.name(iter), ...
%         char(177) + string(sensorsO1.maxDP(iter)), ...
%         string(sensorsO1.logFreq) + " hZ"];
% end %for iter = 1:length(sensorsO1.name)
% for iter = 1:length(sensorsO2.name)
%     tab.data(length(sensorsO1.name) + iter, :) = [sensorsO2.name(iter), ...
%         char(177) + string(sensorsO2.maxDP(iter)), ...
%         string(sensorsO2.logFreq) + " hZ"];
% end %for iter = 1:length(sensorsO2.name)
% 
% currData = [Q_ave; Pr_ave];
%plotQvsPr(readFilename, legName, titleName, tab, currData)



for i = 1:length(sensorsO1.name)
    fprintf("Orifice 1, \t\t sensor %s, \t\t Mean DP: %f\n", ...
        sensorsO1.name(i), mean(sensorsO1.DP(i,:)) );
    sensorsO1.meanDP(i) = mean(sensorsO1.DP(i,:));
end %for i = 1:length(sensorsO1.name)
for i = 1:length(sensorsO2.name)
    fprintf("Orifice 2, \t\t sensor %s, \t\t Mean DP: %f\n", ...
        sensorsO2.name(i), mean(sensorsO2.DP(i,:)) );
    sensorsO2.meanDP(i) = mean(sensorsO2.DP(i,:));
end %for i = 1:length(sensorsO2.name)

%% Average disparity between orifice plates
%This section should attempt to look at how the average disparity between
%data-recordings in the two orifice plates can be stored (saved), so that a
%correction factor may be easily found and updated.

% Should include a way to define the interval which should be looked at.
startTime = pxiTime(1); %Maybe it is better to simply write "0"?
endTime = pxiTime(end); %The end of the current interval
newInterval = 0; %set to 1 if relevant

while newInterval ~= 0
    timeInterval = [startTime, endTime]; %This can be adjusted later

    limited_sensorsO1 = LimitStruct(sensorsO1, timeInterval);
    limited_sensorsO2 = LimitStruct(sensorsO2, timeInterval);
    
    fieldName = fieldnames(limited_sensorsO1);
    
    for i = 1 : length(fieldName)
        if size(sensorsO1.(fieldName{i})) == size(sensorsO2.(fieldName{i}))
            averageDisparity.(fieldName{i}) = limited_sensorsO1.(fieldName{i}) ...
                - limited_sensorsO2.(fieldName{i});
        end %if size(sensorsO1) == size(sensorsO2)
    end %for i = length(sensorsO1)
    
    
    intervalCheck = input("Another interval? (y/n)\n", 's');
    if intervalCheck == 'n'    
        newInterval = 0;
        saveInt = input("Save current interval? (y/n)\n", 's');
        if saveInt == "y"
            %SaveData(saveTitle, data, timeInterval) 
            %Data extraction not quite finished
        end %saveInt == "y"
    elseif intervalCheck == 'y'
        startTime = input("Start time: ");
        endTime = input("End time: ");
    else; error("No valid answer given");
    end %if intervalCheck
end %while newInterval ~= 0

%% Sensor-specific plots and calculations (correction factor included)
%corrFactor = 0.029459958862728; %constant correction factor (in another script)

%for i = 1:length(sensorsO2.name) %Plotting Q for all sensors at orifice 2
%     tit = "Orifice 2, sensor " + sensorsO2.name(i);
% 
%     figure
%     plot(sensorsO2.Q(i,:), Pr)    
% 	title(tit);
% 
%     xlabel("Volumetric flow, $Q$, $(m^3/s)$")
%     ylabel("Pressure ratio, $Pr$, (-)")
%     grid on
%     grid minor
% 
% end %for i =1:length(sensorsO2.name)

%% %% Offset plots

tit = "$Q$ offset between average orifice 1 and individual orifice 2 sensors";
x = linspace(-0.1, 0.8, 10);

for i = 1:length(sensorsO2.name)
    figure
    plot(mean(sensorsO1.Q), sensorsO2.Q(i,:))
    hold on
    plot(x, x, 'k--');
    ylab = "Orifice 2, " + sensorsO2.name(i);
    title(tit);     xlabel("Orifice 1");    ylabel(ylab);
    grid on;    grid minor;
end %for i = 1:length(sensorsO1.name)

figure %offset 2, comparing Orifice 2 sensors against average Orifice 1 values
tit = "$Q$ offset between individual Or.2 sensors, and average Or.1 sensors";
leg = strings(1, length(sensorsO2.name));
maxValue = max(max(sensorsO1.Q));
for i = 1:length(sensorsO2.name)
    plot(mean(sensorsO1.Q), sensorsO2.Q(i,:))
    hold on
    leg(i) = sensorsO2.name(i) + " orifice 2";
end %for i = length(sensorsO2.name)
plot(x, x, 'k--')
title(tit);     xlabel("Orifice 1");    ylabel("Orifice 2");
grid on;    grid minor;
legend(leg, 'location', 'best');

figure
tit = "Average $Q$ offset between Or. 1 and Or. 2";
plot(mean(sensorsO1.Q), mean(sensorsO2.Q))
hold on
plot(mean(sensorsO1.Q), sensorsO2.Q(2,:))
plot(x, x, 'k--')
title(tit);     grid on;    grid minor;
xlabel("Orifice 1, $Q$ ($m^3/s$)");    ylabel("Orifice 2, , $Q$ ($m^3/s$)");
legName = ["Orifice averages", "Orifice 2, DP 50 sensor", "Normal line"];
legend(legName, 'location', 'best');

%% UNCERTAINTY ANALYSIS
uncertaintyAnalysis = 1;
if uncertaintyAnalysis == 1
k = 1.960; %convergance factor. k = 1.960 = 95% confidence


%sensorsO2 uncertainty analysis
sensorsO2.Udp = [1.0112; 2.0224; 10.1119]; %mbar, calculated beforehand
sensorsO2.Ubeta = 2.5251e-3;
%preallocating
sensorsO2.Urho = zeros(length(sensorsO2.name), length(pxiTime));
sensorsO2.Ue = sensorsO2.Urho;
sensorsO2.Uc = sensorsO2.Urho;
sensorsO2.UD = 1e-3; %m
sensorsO2.Ud = 0.1302e-3; %m
sensorsO2.UQ = zeros(length(sensorsO2.name), length(pxiTime));
for i = 1:length(sensorsO2.name) -1
    %preallocating
    scE_beta = zeros(1, length(pxiTime)); 
    scE_p1 = scE_beta;               scE_dp = scE_beta;
    scC_beta = zeros(1, length(pxiTime));
    scC_Re_D = scC_beta;
    Uc = zeros(1, length(pxiTime));
    
for j = 1:length(pxiTime)
    %Expanded uncertainties of parameters
    Up1 = 0.0277e2; %Pa
    UT = 0.0412; %K
    Ubeta = sensorsO2.Ubeta;
    Udp = sensorsO2.Udp(i);

    %simplifying constants
    A = (0.351 + 0.256.*beta2^4 + 0.93 .*beta2^8);

    %Sensitivity coefficients
    scE_beta(j) = 7.440*(0.138 .* beta2.^3 + 0.138.*beta2.^7) .* ...
        ((1 - (abs(sensorsO2.DP(i,j))./p1O2(j))^(1./k) ) - 1);
    scE_p1(j) = ( A .* abs(sensorsO2.DP(i,j)) .* ...
        (1-(abs(sensorsO2.DP(i,j))./p1O2(j)).^(1./k) ) )./ ...
        (k * p1O2(j) .* (p1O2(j) - abs(sensorsO2.DP(i,j)) ) );
    scE_dp(j) = ( A .* (1-(abs(sensorsO2.DP(i,j))./p1O2(j)).^(1./(k-1)) ) )./ ...
        (k * p1O2(j));

    scC_beta(j) = 0.0522 * beta2 - 1.728 * beta2^7 + ...
        3.647e-4 .* beta2.^(-0.3) .* (10.^6./sensorsO2.Re_D(i,j)).^(0.7) + ...
        (0.0658 + 0.0221 .* sensorsO2.A(i,j)) .* ...
        beta2^(2.5) .* (10.^6 ./ sensorsO2.Re_D(i,j)).^(0.3);
    scC_Re_D(j) = -3.6470e-4 .* (10^6 .* beta2).^(0.7).* ...
        sensorsO2.Re_D(i,j).^(-1.7) - ...
        (0.0056 + 0.0019.*sensorsO2.A(i,j)) .* ...
        beta2^(3.5).*(10^6)^(0.3) .* sensorsO2.Re_D(i,j).^(-1.3);

    %Adding the 
    sensorsO2.Urho(i,j) = k .* sqrt( (Up1./k .*1./(R0.*mean(TempOrifice2))).^2 +...
        (UT./k * (-p1O2(j)./(R0 .* mean(TempOrifice2).^2)) ).^2 );

    sensorsO2.Ue(i,j) = k * sqrt( (Ubeta/k * scE_beta(j))^2 + ...
        (Up1 / k * scE_p1(j))^2 + (Udp / k * scE_dp(j))^2 + ...
        (3.5 *sensorsO2.DP(i,j)/(kappa * p1O2(j)) *sensorsO2.epsilon(i,j)*...
        1/k * 1/100 )^2 );
    %calculating Uc. Alternating between two equations, depending on
    %current Re_D

    URe_D = 0; %Assuming (for now) that Reynolds number is known without
        %error, due to the while-loop earlier in the code. This is of
        %course a simplification. 
    switch sensorsO2.Re_D(i, j)
        case sensorsO2.Re_D(i, j) >= 10000
            Uc(j) = k * sqrt( (Ubeta/k .* scC_beta(j)).^2 + ...
                (URe_D/k .* scC_Re_D(j)).^2 + ...
                (0.5 .* sensorsO2.C(i,j) .* 1/k .* 1/100).^2 );
        case sensorsO2.Re_D(i, j) < 10000
            Uc(j) = k * sqrt( (Ubeta/k .* scC_beta(j)).^2 + ...
                (URe_D/k .* scC_Re_D(j)).^2 + ...
                (1 .* sensorsO2.C(i,j) .* 1/k .* 1/100).^2 );
        otherwise
            Uc(j) = k * sqrt( (Ubeta/k .* scC_beta(j)).^2 + ...
                (URe_D/k .* scC_Re_D(j)).^2 + ...
                (0.5 .* sensorsO2.C(i,j) .* 1/k .* 1/100).^2 );
            %error("Case-statements not logical");
    end %switch
    sensorsO2.Uc(i,j) = Uc(j);
sensorsO2.UmP(i,j) = sqrt( (sensorsO2.Uc(i,j)./sensorsO2.C(i,j)).^2 +...
    (sensorsO2.Ue(i,j)./sensorsO2.epsilon(i,j)).^2 + ...
    ( (2 * beta2^4)/(1 - beta2^4) ).^2 .* (sensorsO2.UD ./D_iO2).^2 + ...
    ( 2/(1-beta2^4) ).^2 .* (sensorsO2.Ud ./ d_iO2).^2 + ...
    1/4 .* ( Udp./sensorsO2.DP(i,j) ).^2 + ...
    1/4 .* (sensorsO2.Urho(i, j) ./ sensorsO2.rho(j) ).^2 ); %error given in %/100. From 

sensorsO2.Um(i,j) = sensorsO2.UmP(i,j) .* sensorsO2.m(i,j); %kg/s, expanded uncertainty

sensorsO2.UQ(i,j) = k * sqrt( (sensorsO2.Um(i,j)./k .* 1./sensorsO2.rho(j) ).^2 + ...
    (sensorsO2.Urho(i, j) ./k .* (-sensorsO2.m(i,j)./(sensorsO2.rho(j)).^2 )).^2 ) ;
sensorsO2.UQP(i,j) = sensorsO2.UQ(i,j)/sensorsO2.Q(i,j)*100;
end %for j = length(pxiTime)
legName = sensorsO2.name(i);

figure
plot(sensorsO2.Q(i,:), sensorsO2.UQ(i,:), 'x')
hold on
grid on
grid minor
xlabel("Volumetric flow, Q ($m^3/s$)")
ylabel("Expanded uncertainty, $U_Q$ ($m^3/s$)")
legend(legName, 'location', 'best');

figure
plot(sensorsO2.Q(i,:), sensorsO2.UQP(i,:), 'rx')
hold on
grid on
grid minor
xlabel("Volumetric flow, Q ($m^3/s$)")
ylabel("Relative expanded uncertainty, $U_Q$ ($\%$)")
legend(legName, 'location', 'best');
end %for i = length(sensorsO1.name)

end %if uncertaintyAnalysis == 1;

%% Functions
function SaveData(saveTitle, saveData, saveFilename, sheet)
    %this function saves the data in saveData to an excel sheet named
    %saveFilename, with the title saveTitle in the column left of saveData.
    %saveData must be a matrix with different parameters for each row, and
    %different values for a given parameter is found in the columns
    %sheet contains the string denoting which sheet the data should be
    %saved to
[sRow, ~] = size(saveData);
    %The data to be plotted:
    T = table(saveTitle', saveData');
    
    %Must ensure no such title already exists, and if it does; replace the
    %contents.
    startCol = 0; %Preallocating
    Tr = readtable(saveFilename); %r or _r represents a value read from the 
                                %excel document saveFilename
	[~, col_r] = size(Tr);
    for iter = 1:(sRow + 1):col_r
        %By skipping sRow + 1 (to make room for title) and going all the
        %way to the last read column, iter will loop through all titles in
        %the saved document.
        if (string(Tr{1, iter}) == T{1,1} && string(Tr{2, iter}) == T{2,1})
            startCol = iter-1;
            break;
        else; startCol = col;
        end %if Tr == T
    end %for iter = col
    if iter == 0; startCol = col; end %if iter == 0
    if sheet == 0
        writetable(T, saveFilename, 'Sheet', 1, 'Range', (char('A' + startCol) + "1") )
    else
        writetable(T, saveFilename, sheet, 'Range', (char('A' + startCol) + "1") )
    end %if sheet == 0
end %function SaveData

function returnStruct = LimitStruct(inputStruct, timeInterval)
    %a function that takes in a sensor-struct and a time-interval vector,
    %and returns the struct parameters from within the time-interval.
    elemNames = fieldnames(inputStruct); %imports the field-names of inputStruct
    missingElemNames = elemNames([3:5, 9]); %saving the elemNames that are removed below
    elemNames = elemNames([1:2, 6:8, 10:end]); %removes elements that aren't 
                    %parameters. As it makes no sense to keep them 
                        %(and they mess up the code)
	
    returnStruct = struct;
    for j = 1:length(elemNames)
        [row, ~] = size(inputStruct.(elemNames{j})); %find out how many rows are in the struct.elemName
        for k = 1:row
            returnStruct = setfield(returnStruct, elemNames{j}, pi * ones(row, 1));
            %All first elements of the return struct is set to pi, because
            %pi is an irrational number, and thus one may confidently say
            %it will never appear in this setting "on accident", wheras
            %getting 0 is much more likely. Note that pi is not used for
            %anything but a check to see whether or not the first element
            %has been (rightfully) changed.
        end %for k = row
    end %for j = length(elemNames)
    
    for i = 1:length(inputStruct.time)
        for j = 1:length(elemNames)
            if (inputStruct.time(i) >= timeInterval(1) || inputStruct.time(i) <= timeInterval(2))
                %checks if inside interval. If inside, add to returnStruct
                [row, ~] = size(inputStruct.(elemNames{j})); %find out how many rows are in the struct.elemName
                for k = 1:row
                    %The following if-sentence sets returnStruct to the 
                        %corresponding input struct, if conditions are met
                        %These conditions are: 
                    if returnStruct.(elemNames{j})(k, end) == pi || k ~= 1 
                        % it's the first element (and k is not 1. The
                        % reason that k was included here, is because a new
                        % element will be added if the other if-sentance
                        % fires
                        returnStruct.(elemNames{j})(k, end) = ...
                            inputStruct.(elemNames{j})(k,i);
                    elseif (returnStruct.(elemNames{j})(k, end) ~= pi) && ...
                            (length(inputStruct.(elemNames{j})(k,:)) >= i) && ...
                            (k == 1)
                        %It is not the first element, but there are still
                        %more elements left that need to be set (and k is
                        %1, this will add another element)
                        returnStruct.(elemNames{j})(k, end + 1) = ...
                            inputStruct.(elemNames{j})(k,i);
                    end % if i == 1;
                end% for k = row
            end%if inside interval
        end %for j = length(elemNames)
    end %for i = length(struct.time)
    
    for iter = 1:length(missingElemNames) %re-adding all "removed" elements
        returnStruct.(missingElemNames{iter}) = ...
            inputStruct.(missingElemNames{iter});
    end %for iter = length(missingElemNames)
end%function LimitStruct

function plotQvsPr(readFilename, legName, titleName, figTable, currentData)
    %Plots QvsPr graphs
    %readFilename is the file from which the data in the Q vs Pr plot is 
    %to be read
    %legName is a string containing the name of the current plot
    % (only compatible if there's no table in the figure), or ""
    %titleName is a string containing the title of the figure.
    %If figTable exists, create a new figure, and add the table as one
    %window in the figure. figTable must be a struct with the fields
    %"filename", a string containing either nothing (""), or the filename
    %in which the other fields can be read from (if containing nothing,
    %no other parameters are necessary);
    %"data", a matrix containing the data wanted displayed; "cNames", a
    %string vector containing the column names; "rNames", a string vector
    %containing the row names
    if isstruct(figTable)
        xPos = 100; yPos = 2*xPos; xSize = 1400; ySize = 500;
        f = figure('Position', [xPos, yPos, xSize, ySize]);
        tiledlayout(1,2);
        nexttile%([1,2])
        Q_ave = currentData(1,:); Pr_ave = currentData(2,:);
        plot(Q_ave, Pr_ave)
        hold on

    end % if tableFig
    Tr = readtable(readFilename); [~, col] = size(Tr);
        % Continuation of Q vs Pr plot
    i = 0; %reusing this as an iteration variable
    name = strings(1, col/3); %preallocating
    %name(1) = saveTitle(1) + " " + saveTitle(2); %used for legend, maybe
        %not relevant?
    for n = 1:3:col
        i = i + 1;
        hold on
        Qr = Tr{:, n + 1};
        Pr_r = Tr{:, n + 2};
        name(i) = ( string(Tr{1, n}) + " " + string(Tr{2, n}) );
        plot(Qr, Pr_r)
    end

    %Making the figure "pretty"
    xlabel('Volumetric flow, $Q$, $(m^3/s)$');
    ylabel('Pressure ratio, $Pr$, (-)');
    title(titleName);%, Ueland data');
    hold on
    grid on
    grid minor

    if strlength(legName) > 1 %1 because a null-string always has length 1 
                 %(and a name consisiting of 1 letter shouldn't count anyways)
        name = [legName, name];
    end %if legName < 1
    legend(name, 'location', 'best');
    
    if isstruct(figTable) %Adding the figure table in the second tile
        %nexttile
        if figTable.filename == "" %if a filename does not exist, create the table using figTable.data
            T = table(figTable.data(:,1), figTable.data(:,2), ...
                figTable.data(:,3), 'RowNames', figTable.rNames);
            T.Properties.VariableNames = figTable.cNames;
            %Code below borrowed from:
%https://www.mathworks.com/matlabcentral/answers/254690-how-can-i-display-a-matlab-table-in-a-figure
    %____________________________________%
            % Get the table in string form.
            TString = evalc('disp(T)');
            % Use TeX Markup for bold formatting and underscores.
            TString = strrep(TString,'<strong>','\bf');
            TString = strrep(TString,'</strong>','\rm');
            TString = strrep(TString,'_','\_');
            % Get a fixed-width font.
            FixedWidth = get(0,'FixedWidthFontName');
            % Output the table using the annotation command.
            annotation(gcf,'Textbox','String',TString,'Interpreter','Tex',...
                'FontName',FixedWidth,'Units','Normalized', ...
                'Position',[0.5 0.5 0.4 0.5]);
    %____________________________________%
        else
            %Not implemented yet; Reading table-values from file
        end %if figTable.filename
    end% if figTable, second
end % function plotQvsPr

function [negativeFactor, returnMatrix] = ReversedFlowDetector(inputMatrix)
%This function tries to examine the input matrix inputMatrix for negative
%numbers. If a negative number is found, the position is recorded in the
%matrix negativeFactor, and then changed to the absolute value instead. The
%negativeFactor-matrix is a matrix consisting of either -1 or 1.
    [row, col] = size(inputMatrix);
    negativeFactor = ones(row, col); %preallocating
    for i = 1:row %Loops through the rows of the matrix
        for j = 1:col %Loops through the columns of the matrix
            if inputMatrix(i,j) < 0 %If a value less than 0 is found,
                negativeFactor(i,j) = -1;
            end
        end %for j = col
    end%for i = row
    returnMatrix = abs(inputMatrix); %The return matrix is set as the 
                                     %absolute value of the input matrix
end %function ReversedFlowDetector

function plotTime (yData, yLabel, Title, time)
    plot(time, yData)
    title(Title);
    xlabel('Time, s');
    ylabel(yLabel);
    grid on
    %grid minor
end %func plotTime

function Cp = CpVarianceFunc(Temp, R)
%Calculating Cp variance according to K. Wark, Thermodynamics, 4th edition,
%McGraw-Hill, New York, 1983, as based on NASA SP-273, U.S. Government
%Printing Office, Washington DC, 1971
%Note: Only valid for temperatures between 300 K and 1000 K (so might be
%inaccurate for lowest temp values in this system). Also, variation is
%calculated for selected Ideal Gases
alpha = 3.653;    beta = - 1.337*10^(-3);    gamma = 3.294*10^(-6);
delta = -1.913*10^(-9);    epsilon = 0.2763*10^(-12);
Cp = (alpha + beta .* Temp + gamma .* Temp.^2 +...
    delta .* Temp.^3 + epsilon .* Temp.^4)*(R);
%R is given in kJ/kgK
end % func CpVarianceFunc 

function [dataX, dataY] = jaggedDataSmoother(inputDataX, inputDataY, dyTolerance)
%Purpose: Averages jagged data, so smooth readings are possible (for
%trends). Whenever the derivative of the function becomes a positive number
%greater than dyTolerance, the data recorded up to this point, from the
%previous point gets averaged to one single value. 
%First and last datapoints added to get a line covering the entire
%spectrum. First and last datapoints are thus not "average" values of an
%interval, but actual, recorded values.
    dataX = zeros(1, length(inputDataX)); %Preallocating
    dataY = zeros(1, length(inputDataY)); %Preallocating
    j = 2; % Stores the data at the correct index. data(1) is first element.
        %As explained above, that means it is a "real value"
    n = 1; % Used to determine the length of an interval.
    
    dataX(1) = inputDataX(1);   dataY(1) = inputDataY(1); %Adding first datapoints
    for i = 2:1:length(inputDataX)
        dataY(j) = inputDataY(i) + dataY(j);
        dataX(j) = inputDataX(i) + dataX(j);
        n = n + 1;
        
        if ((inputDataY(i) - inputDataY(i-1)) > dyTolerance || i == length(inputDataX))
            % if the new value is bigger than the previous + the tolerance,
            % or in other words, if the derivative between the data points
            % is larger than the tolerance (and positive), this
            % if-statement fires.
            dataY(j) = dataY(j)/n;
            dataX(j) = dataX(j)/n;
            
            j = j + 1;
            n = 0;
            
        end %if ...
    end %for i = inputDataX
    
    dataX(j) = inputDataX(end);     dataY(j) = inputDataY(end); %adds last data point
    j = j + 1; %Add 1 to add last data point. Ensures correct number of "0"-s
        %is deleted.

    iter = 0; %Counts number of while-loop iterations used to delete elements.
            % Acts as a safeguard.f
    while length(dataY) >= j
        iter = iter + 1;
        dataY(j) = [];
        dataX(j) = [];
        if iter > 10000
            error("Too many iterations in while-loop (over 1000)\n");
        end %if dataY(k) == 0
    end %for k = length(dataX)

end %function dataAverager